      MODULE cg

      use lda_data
      use lda_interfaces

      implicit none

      real(8)      :: CGconv
      integer      :: iterCG
      character(6) :: CG_Solver

      INTERFACE ConjGrad
         MODULE PROCEDURE R_ConjGrad,C_ConjGrad
      END INTERFACE

      CONTAINS

c=======================================================================

      SUBROUTINE R_ConjGrad( Xn, B0, E )

      real(8),intent(OUT) :: Xn(ML)
      real(8),intent(IN)  :: B0(ML)
      real(8),intent(IN)  :: E
      real(8) :: b2,r2
      real(8) :: an,bn
      real(8) :: Rn(ML),Pn(ML),B(ML),ovrp(MST)
      real(8) :: rr,rr_0
      real(8),allocatable :: APn(:)              ! CG
      real(8),allocatable :: R0(:),Un(:),Zn(:)   ! CGS
      real(8),allocatable :: RRn(:),PPn(:)       ! BiCG
!      real(8) :: A(ML),Q(ML)            ! CR
!      real(8)    :: mu                     !
!      real(8) :: Tn(ML),Tn0(ML),Wn(ML),Yn(ML),AT(ML),AP(ML) ! GPBiCG
!      real(8) :: zeta_n,eta_n,yAt,Aty,Att,At2,y2,yt         !

      real(8) :: tpsi(ML),htpsi(ML)
      integer,parameter :: maxiter=5000
      integer :: p,iter

c Projection

      B = B0
      do p=1,MST
         ovrp(p)=sum( psi(:,p)*B0 )*H**3
         B=B-psi(:,p)*ovrp(p)
      end do

c Ini. 

      b2=sum(B*B)

      Xn=0.d0  ! ; call hpsi(Xn,Rn) ; Rn=B-E*Xn+Rn
      Rn=B

      r2=sum(Rn*Rn) ; bn=1.d0/r2 ; Pn=bn*Rn

c Iteration

      select case( CG_Solver )

c CGS
      case('CGS') ! R0,Rn,rr,rr_0,bn,Zn,Un,Pn,htpsi,r2

         allocate( R0(ML),Un(ML),Zn(ML) )

         R0=Rn ; rr_0=sum(R0*Rn) ; bn=0.d0 ; Un=0.d0 ; Zn=Un
         do iter=1,maxiter
            Pn=Rn+bn*Zn
            Un=Pn+bn*(Zn+bn*Un) ; call hpsi(Un,htpsi)
            an=rr_0/sum(R0*(E*Un-htpsi))
            Zn=Pn-an*(E*Un-htpsi)
            Xn=Xn+an*(Pn+Zn)
            tpsi=Pn+Zn ; call hpsi(tpsi,htpsi)
            Rn=Rn-an*(E*tpsi-htpsi)
            rr=sum(R0*Rn) ; bn=rr/rr_0 ; rr_0=rr
            r2=sum(Rn*Rn) ; if ( r2/b2 < CGconv ) Exit
         end do

         deallocate( R0,Un,Zn )

c Bi-CG
      case('BiCG') ! bn,RRn,Rn,rr,rr_0,Pn,PPn,an,htpsi,r2

         allocate( RRn(ML),PPn(ML) )

         bn=0.d0 ; RRn=Rn ; rr_0=sum(RRn*Rn)
         Pn=0.d0 ; PPn=Pn
         do iter=1,maxiter
            Pn=Rn+bn*Pn
            PPn=RRn+bn*PPn
            call hpsi(Pn,htpsi)
            an=rr_0/sum(PPn*(E*Pn-htpsi))
            Xn=Xn+an*Pn ; Rn=Rn-an*(E*Pn-htpsi)
            call hpsi(PPn,htpsi)
            RRn=RRn-an*(E*PPn-htpsi)
            rr=sum(RRn*Rn) ; bn=rr/rr_0 ; rr_0=rr
            r2=sum(Rn*Rn) ; if ( r2/b2 < CGconv ) Exit
         end do

         deallocate( RRn,PPn )

c CG
      case('CG')

         allocate( APn(ML) )

         do iter=1,maxiter
            call hpsi(Pn,APn) ; APn=E*Pn-APn
            an=1.d0/sum( Pn*APn ) ; Xn=Xn+an*Pn ; Rn=Rn-an*APn
            r2=sum(Rn*Rn) ; if ( r2/b2 < CGconv ) Exit
            bn=1.d0/r2 ; Pn=Pn+bn*Rn
         end do

         deallocate( APn )

c CR
      case('CR') ! Pn,Rn,Q,mu,an,A,htpsi,bn

!      Pn=Rn
!      call hpsi(Pn,htpsi) ; Q=E*Pn-htpsi
!      do iter=1,maxiter
!         mu=sum(Q*Q) ; an=sum(Rn*Q)/mu
!         Xn=Xn+an*Pn ; Rn=Rn-an*Q
!         r2=sum(Rn*Rn) ; if ( r2/b2 < CGconv ) Exit
!         call hpsi(Rn,htpsi) ; A=E*Rn-htpsi
!         bn=-sum(A*Q)/mu ; Pn=Rn+bn*Pn ; Q=A+bn*Q
!      end do

c GPBi-CG
      case('GPBiCG') ! bn,R0,Rn,rr_0,Tn,Tn0,Un,Zn,Pn,htpsi,AP,an,Yn,AT
                     ! y2,yt,Att,Aty,At2,yAt,zeta_n,eta_n,Wn
!
!      bn=0.d0 ; R0=Rn ; rr_0=sum(R0*Rn)
!      Tn=0.d0 ; Tn0=Tn ; Wn=Tn ; Un=Tn ; Zn=Tn ; Pn=Tn
!      do iter=1,maxiter
!         Pn=Rn+bn*(Pn-Un) ; call hpsi(Pn,htpsi) ; AP=E*Pn-htpsi
!         an=rr_0/sum(R0*AP)
!         Yn=Tn-Rn-an*Wn+an*AP
!         Tn=Rn-an*AP ; call hpsi(Tn,htpsi) ; AT=E*Tn-htpsi
!         y2=sum(Yn*Yn)    ; yt=sum(Yn*Tn)
!         Att=sum(AT*Tn) ; At2=sum(AT*AT)
!         Aty=sum(AT*Yn) ; yAt=sum(Yn*AT)
!         if(iter==1)then
!            zeta_n=Att/At2 ; eta_n=0.d0
!         else
!            zeta_n=(y2*Att-yt*Aty)/(At2*y2-yAt*Aty)
!            eta_n =(At2*yt-yAt*Att)/(At2*y2-yAt*Aty)
!         end if 
!         Un=zeta_n*AP+eta_n*(Tn0-Rn+bn*Un) ; Zn=zeta_n*Rn+eta_n*Zn-an*Un
!         Xn=Xn+an*Pn+Zn ; Rn=Tn-eta_n*Yn-zeta_n*AT
!         rr=sum(R0*Rn) ; bn=an/zeta_n*rr/rr_0 ; rr_0=rr
!         Wn=AT+bn*AP ; Tn0=Tn
!         r2=sum(Rn*Rn) ; if ( r2/b2 < CGconv ) Exit
!      end do

      end select

      if ( iter > maxiter ) then
         write(*,*) ' iteration not converged ( CG )   r2/b2 =',r2/b2
      end if

      ovrp=ovrp/(E-esp)
      do p=1,MST
         Xn=Xn+ovrp(p)*psi(:,p)
      end do

      iterCG=iterCG+iter

      return

      END SUBROUTINE R_ConjGrad

c=======================================================================

      SUBROUTINE C_ConjGrad( Xn, B0, E )

      complex(8),intent(OUT) :: Xn(ML)
      complex(8),intent(IN)  :: B0(ML)
      complex(8),intent(IN)  :: E
      real(8)    :: b2,r2
      complex(8) :: an,bn
      complex(8) :: Rn(ML),Pn(ML),B(ML),ovrp(MST)
      complex(8) :: rr,rr_0
      complex(8),allocatable :: APn(:)              ! CG
      complex(8),allocatable :: R0(:),Un(:),Zn(:)   ! CGS
      complex(8),allocatable :: RRn(:),PPn(:)       ! BiCG
!      complex(8) :: A(ML),Q(ML)            ! CR
!      real(8)    :: mu                     !
!      complex(8) :: Tn(ML),Tn0(ML),Wn(ML),Yn(ML),AT(ML),AP(ML) ! GPBiCG
!      complex(8) :: zeta_n,eta_n,yAt,Aty,Att,At2,y2,yt         !

      complex(8) :: tpsi(ML),htpsi(ML)

      integer,parameter :: maxiter=5000
      integer :: p,iter

c Projection

      B = B0
      do p=1,MST
         ovrp(p)=sum( psi(:,p)*B0 )*H**3
         B=B-psi(:,p)*ovrp(p)
      end do

c Ini. 

      b2=sum( abs(B)**2 )

      Xn=(0.d0,0.d0)  ! ; call hpsi(Xn,Rn) ; Rn=B-E*Xn+Rn
      Rn=B

      r2=sum( abs(Rn)**2 ) ; bn=1.d0/r2 ; Pn=bn*Rn

c Iteration

      select case( CG_Solver )

c CGS
      case('CGS') ! R0,Rn,rr,rr_0,bn,Zn,Un,Pn,htpsi,r2

         allocate( R0(ML),Un(ML),Zn(ML) )

         R0=conjg(Rn) ; rr_0=sum(R0*Rn) ; bn=(0.d0,0.d0)
         Un=(0.d0,0.d0) ; Zn=Un
         do iter=1,maxiter
            Pn=Rn+bn*Zn
            Un=Pn+bn*(Zn+bn*Un) ; call hpsi(Un,htpsi)
            an=rr_0/sum(R0*(E*Un-htpsi))
            Zn=Pn-an*(E*Un-htpsi)
            Xn=Xn+an*(Pn+Zn)
            tpsi=Pn+Zn ; call hpsi(tpsi,htpsi)
            Rn=Rn-an*(E*tpsi-htpsi)
            rr=sum(R0*Rn) ; bn=rr/rr_0 ; rr_0=rr
            r2=sum(abs(Rn)**2) ; if ( r2/b2 < CGconv ) Exit
         end do

         deallocate( R0,Un,Zn )

c Bi-CG
      case('BiCG') ! bn,RRn,Rn,rr,rr_0,Pn,PPn,an,htpsi,r2

         allocate( RRn(ML),PPn(ML) )

         bn=(0.d0,0.d0) ; RRn=Rn ; rr_0=sum(conjg(RRn)*Rn)
         Pn=(0.d0,0.d0) ; PPn=Pn
         do iter=1,maxiter
            Pn=Rn+bn*Pn
            PPn=RRn+conjg(bn)*PPn
            call hpsi(Pn,htpsi)
            an=rr_0/sum(conjg(PPn)*(E*Pn-htpsi))
            Xn=Xn+an*Pn ; Rn=Rn-an*(E*Pn-htpsi)
            call hpsi(PPn,htpsi)
            RRn=RRn-conjg(an)*(conjg(E)*PPn-htpsi)
            rr=sum(conjg(RRn)*Rn) ; bn=rr/rr_0 ; rr_0=rr
            r2=sum(abs(Rn)**2) ; if ( r2/b2 < CGconv ) Exit
         end do

         deallocate( RRn,PPn )

c CG
      case('CG')

         allocate( APn(ML) )

         do iter=1,maxiter
            call hpsi(Pn,APn) ; APn=E*Pn-APn
            an=1.d0/sum( Pn*APn ) ; Xn=Xn+an*Pn ; Rn=Rn-an*APn
            r2=sum(Rn**2) ; if ( r2/b2 < CGconv ) Exit
            bn=1.d0/r2 ; Pn=Pn+bn*Rn
         end do

         deallocate( APn )

c CR
      case('CR') ! Pn,Rn,Q,mu,an,A,htpsi,bn

!      Pn=Rn
!      call hpsi(Pn,htpsi) ; Q=E*Pn-htpsi
!      do iter=1,maxiter
!         mu=sum(abs(Q)**2) ; an=sum(conjg(Rn)*Q)/mu
!         Xn=Xn+an*Pn ; Rn=Rn-an*Q
!         r2=sum(abs(Rn)**2) ; if ( r2/b2 < CGconv ) Exit
!         call hpsi(Rn,htpsi) ; A=E*Rn-htpsi
!         bn=-sum(conjg(A)*Q)/mu ; Pn=Rn+bn*Pn ; Q=A+bn*Q
!      end do

c GPBi-CG
      case('GPBiCG') ! bn,R0,Rn,rr_0,Tn,Tn0,Un,Zn,Pn,htpsi,AP,an,Yn,AT
                     ! y2,yt,Att,Aty,At2,yAt,zeta_n,eta_n,Wn
!
!      bn=(0.d0,0.d0) ; R0=conjg(Rn) ; rr_0=sum(R0*Rn)
!      Tn=(0.d0,0.d0) ; Tn0=Tn ; Wn=Tn ; Un=Tn ; Zn=Tn ; Pn=Tn
!      do iter=1,maxiter
!         Pn=Rn+bn*(Pn-Un) ; call hpsi(Pn,htpsi) ; AP=E*Pn-htpsi
!         an=rr_0/sum(R0*AP)
!         Yn=Tn-Rn-an*Wn+an*AP
!         Tn=Rn-an*AP ; call hpsi(Tn,htpsi) ; AT=E*Tn-htpsi
!         y2=sum(abs(Yn)**2)    ; yt=sum(conjg(Yn)*Tn)
!         Att=sum(conjg(AT)*Tn) ; At2=sum(abs(AT)**2)
!         Aty=sum(conjg(AT)*Yn) ; yAt=sum(conjg(Yn)*AT)
!         if(iter==1)then
!            zeta_n=Att/At2 ; eta_n=(0.d0,0.d0)
!         else
!            zeta_n=(y2*Att-yt*Aty)/(At2*y2-yAt*Aty)
!            eta_n =(At2*yt-yAt*Att)/(At2*y2-yAt*Aty)
!         end if 
!         Un=zeta_n*AP+eta_n*(Tn0-Rn+bn*Un) ; Zn=zeta_n*Rn+eta_n*Zn-an*Un
!         Xn=Xn+an*Pn+Zn ; Rn=Tn-eta_n*Yn-zeta_n*AT
!         rr=sum(R0*Rn) ; bn=an/zeta_n*rr/rr_0 ; rr_0=rr
!         Wn=AT+bn*AP ; Tn0=Tn
!         r2=sum(abs(Rn)**2) ; if ( r2/b2 < CGconv ) Exit
!      end do

      end select

      if ( iter > maxiter ) then
         write(*,*) ' iteration not converged ( CG )   r2/b2 =',r2/b2
      end if

      ovrp=ovrp/(E-esp)
      do p=1,MST
         Xn=Xn+ovrp(p)*psi(:,p)
      end do

      iterCG=iterCG+iter

      return

      END SUBROUTINE C_ConjGrad

      END MODULE cg
