c=================================================== RPA equation Solver
c============================================(Conjugate-Residual Method)

      SUBROUTINE ConjResidu( Xn, B, hw1 )
      use chyp_data
      implicit none

      complex(8),intent(INOUT) :: Xn(ML)
      complex(8),intent(IN)    :: B(ML)
      complex(8),intent(IN)    :: hw1

      integer,parameter :: max_iter=200  ! max iteration times
!      integer,parameter :: Nrestart=3    ! restart, if irst>Nrestart
!      integer,parameter :: Mrestart=10   ! exit iteration, if trst>Mrestart
!      real(8),parameter :: ep_rst=1.005d0

      integer :: iter,iter0,irst,trst,iterVh,n
      real(8) :: an,bn,b2,dp(3),err_new,err_old
      complex(8) :: Rn(ML),S(ML),trD(ML)
      complex(8),allocatable :: Pn(:,:),Q(:,:),mu(:)

      allocate( Pn(ML,Nrestart),Q(ML,Nrestart),mu(Nrestart) )

      iter=1 ; trst=0 ; err_old=10.d0 ; err_new=1.d0

      write(*,'(1x,4x,3(5x,a4,5x),5x,a4,5x,2a7)')
     &     "<x>","<y>","<z>","err","CG","Vh"

      b2=sum( abs(B)**2 )

c Start & Restart

      Restart : do trst=0,Mrestart

         if (trst>0) write(*,'(/1x,"------------------ Restart -----")')
         irst=1 ; iter0=iter

         if ( all( mask = Xn==(0.d0,0.d0) ) ) then
            Rn=B
         else
            Rn=(0.d0,0.d0) ; call Hartree(Xn,Rn,Hconv)
            Rn=Rn+Fxc*Xn   ; call calc_chi1(trD,Rn,hw1)
            Rn=B-Xn+trD
         end if

         Pn(:,1)=Rn

         Q=(0.d0,0.d0)             ; call Hartree(Pn(:,1),Q(:,1),Hconv)
         Q(:,1)=Q(:,1)+Fxc*Pn(:,1) ; call calc_chi1(trD,Q(:,1),hw1)
         Q(:,1)=Pn(:,1)-trD                           ! Q <--> A*Pn

c Iteration

         Iteration : do iter=iter0,max_iter

            iterCG=0

            mu(irst)=sum(abs(Q(:,irst))**2)
            an=sum(conjg(Rn)*Q(:,irst))/mu(irst)
            Xn=Xn+an*Pn(:,irst) ; Rn=Rn-an*Q(:,irst)

            S=(0.d0,0.d0) ; call Hartree(Rn,S,Hconv,iterVh)
            S=S+Fxc*Rn    ; call calc_chi1(trD,S,hw1)
            S=Rn-trD                                  ! S <--> A*Rn

!            bn=-sum(conjg(S)*Q)/mu ; Pn=Rn+bn*Pn ; Q=S+bn*Q

c Info & Judge

            err_new=sum(abs(Rn)**2)/b2


            dp=const*matmul(Xn,D)*H**3
            write(*,'(1x,i4,3g14.5,e14.4,2i7)')
     &           iter,dp(1),dp(2),dp(3),err_new,iterCG,iterVh

            if ( err_new < ep ) goto 1

!            if ( err_old/err_new > ep_rst ) irst=0
!            err_old=err_new

            irst=irst+1
            if ( irst>Nrestart ) cycle Restart

            Pn(:,irst)=Rn
            Q(:,irst) =S
            do n=1,irst-1
               bn=-sum(conjg(S)*Q(:,n))/mu(n)
               Pn(:,irst)=Pn(:,irst)+bn*Pn(:,n)
               Q(:,irst) =Q(:,irst)+bn*Q(:,n)
            end do


         end do Iteration

         exit Restart

      end do Restart

      write(*,*) "iteration not converged (ConjResidu)"
      write(*,*) "iter =",iter
      write(*,*) "trst =",trst

 1    continue

      deallocate( Pn,Q,mu )

      return

      END SUBROUTINE ConjResidu
