c=======================================================================
c============ Second-order Independent-Particle Response Function (real)
c=================================================== ( non-symmetrized )

      SUBROUTINE R_calc_chi2( trD, tH1,tH2, hw1,hw2 )
      use lda_data
      use cg
      implicit none
      real(8),intent(OUT) :: trD(ML)
      real(8),intent(IN)  :: tH1(ML),tH2(ML)
      real(8),intent(IN)  :: hw1,hw2
      real(8),allocatable :: v1(:),v2(:)
      integer             :: p

      trD=0.d0

      allocate( v1(ML), v2(ML) ) ; v1=0.d0 ; v2=v1

      do p=1,MST

         call ConjGrad( v2, psi(:,p)*tH2, esp(p)+hw2 )
         call ConjGrad( v1, tH1*v2      , esp(p)+hw1+hw2 )

         trD = trD + occ(p)*psi(:,p)*v1

         call ConjGrad( v1, psi(:,p)*tH1, esp(p)-hw1 )

         trD = trD + occ(p)*v1*v2

         call ConjGrad( v2, psi(:,p)*tH2, esp(p)-hw2 )
         call ConjGrad( v1, tH1*v2      , esp(p)-hw1-hw2 )

         trD = trD + occ(p)*psi(:,p)*v1

      end do

      deallocate( v1, v2 )

      return

      END SUBROUTINE R_calc_chi2

c=======================================================================
c========= Second-order Independent-Particle Response Function (complex)
c=================================================== ( non-symmetrized )

      SUBROUTINE C_calc_chi2( trD, tH1,tH2, hw1,hw2 )
      use lda_data
      use cg
      implicit none
      complex(8),intent(OUT) :: trD(ML)
      complex(8),intent(IN)  :: tH1(ML),tH2(ML)
      complex(8),intent(IN)  :: hw1,hw2
      complex(8),allocatable :: v1(:),v2(:)
      integer :: p

      trD=(0.d0,0.d0)

      allocate( v1(ML), v2(ML) ) ; v1=(0.d0,0.d0) ; v2=v1

      do p=1,MST

         call ConjGrad( v2, psi(:,p)*tH2, esp(p)+hw2 )
         call ConjGrad( v1, tH1*v2      , esp(p)+hw1+hw2 )

         trD = trD + occ(p)*psi(:,p)*v1

         call ConjGrad( v1, psi(:,p)*conjg(tH1), esp(p)-conjg(hw1) )

         trD = trD + occ(p)*conjg(v1)*v2

         call ConjGrad( v2, psi(:,p)*conjg(tH2), esp(p)-conjg(hw2) )
         call ConjGrad( v1, conjg(tH1)*v2      , esp(p)-conjg(hw1+hw2) )

         trD = trD + occ(p)*psi(:,p)*conjg(v1)

      end do

      deallocate( v1, v2 )

      return

      END SUBROUTINE C_calc_chi2
