
      MODULE global_variables
      use chyp_data
      implicit none
      integer :: Na,Nb
      END MODULE global_variables

c=======================================================================

      PROGRAM Hypol_Tensor
      use global_variables
      implicit none

      character(20) :: file_IN, process, Hyp_disp
      character(20) :: adisp_data(3)
      character(20) :: bdisp_data(3,3,3)
      character(20) :: gdisp_data(3,3,3)
      character(4)  :: XC_in,dir
      integer :: a,b,c,n,nhw,min_Nhw,max_Nhw,mhw
      real(8) :: dshif(3),dhw1,dhw2
      complex(8) :: hw1_in,hw2_in

c Input

      read(*,*) file_IN
      read(*,*) Hyp_disp
      read(*,*) XC_in
      read(*,*) CG_solver
      read(*,*) process
      read(*,*) dir             ! [ 'x','y','z','yz','xyz' ]
      read(*,*) hw1_in, hw2_in
      read(*,*) dhw1, dhw2, max_Nhw
!      read(*,*) CGconv, Hconv, ep
!      read(*,*) ep_rst
!      read(*,*) Nrestart,Mrestart 

      CGconv=1.d-8
      Hconv=1.d-10
      ep=1.d-12 ; Nrestart=3 ; Mrestart=10 ; ep_rst=1.005d0

      select case(dir)
      case('x')        ; Na=1 ; Nb=1
      case('y')        ; Na=2 ; Nb=2
      case('z')        ; Na=3 ; Nb=3
      case('yz')       ; Na=2 ; Nb=3
      case default     ; Na=1 ; Nb=3
      end select

      mhw=2

c Make file names

      n=len_trim(Hyp_disp)
      do a=Na,Nb
         adisp_data(a)=Hyp_disp(1:n)//"_"//achar(a+48)//".alp"
      end do
      do a=Na,Nb
      do b=Na,Nb
      do c=Na,Nb
         bdisp_data(a,b,c)=Hyp_disp(1:n)//"_"
     &   //achar(a+48)//achar(b+48)//achar(c+48)//".bt"
      end do
      end do
      end do
      do a=Na,Nb
      do b=Na,Nb
      do c=Na,Nb
         gdisp_data(a,b,c)=Hyp_disp(1:n)//"_"
     &   //achar(a+48)//achar(b+48)//achar(c+48)//".gam"
      end do
      end do
      end do

c Read LDA data

      open(4,file=file_IN,form='unformatted',status='old')
        call IN_data
      close(4)

c Allocate

      allocate( D(ML,3), dn1(ML,Na:Nb,mhw) , h1(ML,Na:Nb,mhw)
     &                 , dn2(ML,Na:Nb,Na:Nb,3), h2(ML,Na:Nb,Na:Nb,3) )
      allocate( Fxc(ML), Gxc(ML), Hxc(ML) )

c Dipole Operator

      dshif(1)=sum(Lx(:ML)*rho)*H/sum(rho) ; D(:,1)=Lx(:ML)*H-dshif(1)
      dshif(2)=sum(Ly(:ML)*rho)*H/sum(rho) ; D(:,2)=Ly(:ML)*H-dshif(2)
      dshif(3)=sum(Lz(:ML)*rho)*H/sum(rho) ; D(:,3)=Lz(:ML)*H-dshif(3)

c Exchange-Correlation Kernel

      call Exc_Cor_Kernel(XC_in,Fxc,Gxc,Hxc)

c Initialize

      dn1       = (0.d0,0.d0) ; dn2      = (0.d0,0.d0) ; h1=dn1 ; h2=dn2
      alphaF    = (0.d0,0.d0) ; alpha    = alphaF
      alphaF_av = (0.d0,0.d0) ; alpha_av = alphaF_av
      betaF     = (0.d0,0.d0) ; beta     = betaF
      betaF_av  = (0.d0,0.d0) ; beta_av  = betaF_av
      gammaF    = (0.d0,0.d0) ; gamma    = gammaF
      gammaF_av = (0.d0,0.d0) ; gamma_av = gammaF_av

      hw(1)=hw1_in ; hw(2)=hw2_in ; min_Nhw=0

! Timer On

      call watch

c

      frequency : do nhw=min_Nhw,max_Nhw

      write(*,'(/1x,"hw1, hw2, hw3, eta =",4f10.5)')
     &       real(hw(1)),real(hw(2)),real(hw(3)),abs(imag(hw(1)))
      write(*,*)    "process            = ", process
      write(*,*)    "dir                = ", dir
      write(*,*)    "lda file           = ", file_IN
      write(*,*)    "xc kernel          = ", XC_in
      write(*,'(1x,"CRconv, CGconv, Hconv =",3g12.4)') ep, CGconv, Hconv
      write(*,'(1x,"dshif =",3g14.4/)') dshif(1), dshif(2), dshif(3)

      select case(process)

c---------------------------------------------- EFISH

      case('EFISH','efish')

c alpha

         const=-E2

         n=1
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

         n=2
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

c beta(-2w;w,w)

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c beta(-w;0,w), beta(-w;w,0)

         n=2
         do a=Na,Nb
         do b=Na,Nb
            call calc_beta( n, a,b, 2,1, hw(2),hw(1) )
              dn2(:,b,a,n+1) =   dn2(:,a,b,n)
               h2(:,b,a,n+1) =    h2(:,a,b,n)
            betaF(:,b,a,n+1) = betaF(:,a,b,n)
             beta(:,b,a,n+1) =  beta(:,a,b,n)
            call watch(DISP='on')
         end do
         end do

! gamma_aabb(-2w;0,w,w)

         const = -E2**2

         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma( a,b,b, 2,1,1, 1,3,2, hw(2),hw(1),hw(1) )
            call watch(DISP='on')
         end do
         end do

! gamma_abba(-2w;0,w,w), gamma_abab(-2w;0,w,w)

         do a=Na,Nb
         do b=Na,Nb
            if ( a==b ) cycle
            call calc_gamma( b,b,a, 2,1,1, 1,3,2, hw(2),hw(1),hw(1) )
            gammaF(:,b,a,b)=gammaF(:,b,b,a)
             gamma(:,b,a,b)= gamma(:,b,b,a)
            call watch(DISP='on')
         end do
         end do

c---------------------------------------------- DFWM

      case('DFWM','dfwm','IDRI','idri')

c alpha

         const=-E2

         do n=1,mhw
            do a=Na,Nb
               call calc_alpha( n, a, hw(n) )
               if ( n==1 ) call adisp_result(hw(n))
               call watch(DISP='on')
            end do
         end do

c beta(-2w;w,w) ( SHG )

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c beta(0;w,-w), beta(0;-w,w) ( Optical Rectification )

         n=2
         do a=Na,Nb
         do b=Na,Nb
            call calc_beta( n, a,b, 1,2, hw(1),hw(2) )
              dn2(:,b,a,n+1) =   dn2(:,a,b,n)
               h2(:,b,a,n+1) =    h2(:,a,b,n)
            betaF(:,b,a,n+1) = betaF(:,a,b,n)
             beta(:,b,a,n+1) =  beta(:,a,b,n)
            call watch(DISP='on')
         end do
         end do

c gamma_aabb(-w;w,-w,w), gamma_abba(-w;w,-w,w)

         const = -E2**2

         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma(a,b,b, 1,2,1, 3,1,2, hw(1),hw(2),hw(1))
            gammaF(:,b,b,a)=gammaF(:,a,b,b)
            gamma(:,b,b,a) =gamma(:,a,b,b)
            call gdisp_result(hw(1),a,b,b)
            call watch(DISP='on')
         end do
         end do

c gamma_abab(-w;w,-w,w)

         do a=Na,Nb
         do b=Na,Nb
            if ( a==b ) cycle
            call calc_gamma(b,a,b, 1,2,1, 3,1,2, hw(1),hw(2),hw(1))
            call gdisp_result(hw(1),b,a,b)
            call watch(DISP='on')
         end do
         end do

c----------------------------------------------- THG

      case('THG','thg')

         const=-E2

         n=1
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

c beta(-2w;w,w)

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c gamma(-3w;w,w,w)

         const = -E2**2

         n=1
         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma( a,b,b, 1,1,1, 1,1,1, hw(1),hw(1),hw(1) )
            gammaF(:,b,b,a)=gammaF(:,a,b,b)
             gamma(:,b,b,a)= gamma(:,a,b,b)
            gammaF(:,b,a,b)=gammaF(:,a,b,b)
             gamma(:,b,a,b)= gamma(:,a,b,b)
            call watch(DISP='on')
         end do
         end do

      end select

      alphaF    = (0.d0,0.d0) ; alpha=alphaF
      alphaF_av = (0.d0,0.d0) ; alpha_av=alphaF_av
      betaF     = (0.d0,0.d0) ; beta=betaF
      betaF_av  = (0.d0,0.d0) ; beta_av=betaF_av
      gammaF    = (0.d0,0.d0) ; gamma=gammaF
      gammaF_av = (0.d0,0.d0) ; gamma_av=gammaF_av

      hw(1)=hw(1)+dhw1 ; hw(2)=hw(2)+dhw2

      end do frequency

      stop

      CONTAINS

c-------------------------- disp_results

      subroutine adisp_result(E)
      implicit none
      complex(8),intent(IN) :: E
      open(1,file=adisp_data(a),position='append')
      write(1,'(1x,f10.4,4f15.5)')
     &     real(E),alpha(a,a,n),alphaF(a,a,n)
      close(1)
      return
      end subroutine adisp_result

!      subroutine bdisp_result(a,b,c)
!      implicit none
!      integer,intent(IN) :: a,b,c
!      open(1,file=bdisp_data(a,b,c),position='append')
!      write(1,'(1x,f10.4,4f15.5)')
!     &     real(hw(1)),beta(a,b,c,1),betaF(a,b,c,1)
!      close(1)
!      return
!      end subroutine bdisp_result

      subroutine gdisp_result(E,a,b,c)
      implicit none
      complex(8),intent(IN) :: E
      integer   ,intent(IN) :: a,b,c
      open(1,file=gdisp_data(a,b,c),position='append')
      if ( b==c ) then
         write(1,'(1x,f10.4,4f15.5)')
     &        real(E),gamma(a,a,b,b),gammaF(a,a,b,b)
      else if ( a==b ) then
         write(1,'(1x,f10.4,4f15.5)')
     &        real(E),gamma(c,a,a,c),gammaF(c,a,a,c)
      else
         write(1,'(1x,f10.4,4f15.5)')
     &        real(E),gamma(b,a,b,a),gammaF(b,a,b,a)
      end if
      close(1)
      return
      end subroutine gdisp_result


      END PROGRAM Hypol_Tensor
