c=================================================== RPA equation Solver
c============================================(Conjugate-Residual Method)

      SUBROUTINE ConjResidu( Xn, B, hw1 )
      use hyp_data
      implicit none

      real(8),intent(INOUT) :: Xn(ML)
      real(8),intent(IN)    :: B(ML), hw1

      integer,parameter :: max_iter=200 ! max iteration times
      integer,parameter :: Nrestart=5   ! restart, if irst>Nrestart
      integer,parameter :: Mrestart=10  ! exit iteration, if trst>Mrestart
      real(8),parameter :: ep_rst=1.005d0

      integer :: iter,iter0,irst,trst,iterVh
      real(8) :: mu,an,bn,b2,dp(3),err_new,err_old
      real(8) :: Rn(ML),Pn(ML),Q(ML),A(ML),trD(ML)


      iter=1 ; trst=0 ; err_old=10.d0 ; err_new=1.d0

      write(*,'(1x,4x,3(5x,a4,5x),5x,a4,5x,2a7)')
     &     "<x>","<y>","<z>","err","CG","Vh"

      b2=sum( B*B )

c Start & Restart

      Restart : do trst=0,Mrestart

         if (trst>0) write(*,'(/1x,"------------------ Restart -----")')

         irst=0 ; iter0=iter

         if ( all( mask = Xn==0.d0 ) ) then
            Rn=B
         else
            Rn=0.d0      ; call Hartree(Xn,Rn,Hconv)
            Rn=Rn+Fxc*Xn ; call calc_chi1(trD,Rn,hw1)
            Rn=B-Xn+trD
         end if

         Pn=Rn

         Q=0.d0     ; call Hartree(Pn,Q,Hconv)
         Q=Q+Fxc*Pn ; call calc_chi1(trD,Q,hw1)
         Q=Pn-trD

c Iteration

         Iteration : do iter=iter0,max_iter

            iterCG=0

            mu=sum(Q*Q) ; an=sum(Rn*Q)/mu
            Xn=Xn+an*Pn ; Rn=Rn-an*Q

            A=0.d0     ; call Hartree(Rn,A,Hconv,iterVh)
            A=A+Fxc*Rn ; call calc_chi1(trD,A,hw1)
            A=Rn-trD

            bn=-sum(A*Q)/mu ; Pn=Rn+bn*Pn ; Q=A+bn*Q

c iteration info.

            dp=const*matmul(Xn,D)*H**3
            write(*,'(1x,i4,3g14.5,e14.4,2i7)')
     &           iter,dp(1),dp(2),dp(3),err_new,iterCG,iterVh

            err_new=sum(Rn*Rn)/b2 ; if (err_new < ep) return

            irst=irst+1
            if ( err_old/err_new > ep_rst ) irst=0
            err_old=err_new

            if ( irst==Nrestart ) cycle Restart

         end do Iteration

         exit Restart

      end do Restart

      write(*,*) "iteration not converged (ConjResidu)"
      write(*,*) "iter =",iter
      write(*,*) "trst =",trst

      return

      END SUBROUTINE ConjResidu
