c=======================================================================
c============================ Hartree potential (Solve Poisson equation)

      SUBROUTINE R_Hartree(trho,tVh,Hconv,iterVh)
      use lda_data
      implicit none
      real(8),intent(IN)           :: trho(ML)
      real(8),intent(INOUT)        :: tVh(ML)
      real(8),intent(IN),optional  :: Hconv
      integer,intent(OUT),optional :: iterVh

      integer,parameter :: maxiter=400
      integer :: i,j,ix,iy,iz,lm,L,a,iter
      real(8) :: x,y,z,r,sum1,sum2,ak,ck,xx,yy,zz
      real(8) :: rholm(25,MI),conv
      real(8),allocatable :: zk(:),tk(:),pk(:),wk(:,:,:)
       
      INTERFACE
         FUNCTION Ylm(x,y,z,lm)
         implicit none
         real(8) :: Ylm
         real(8),intent(IN) :: x,y,z
         integer,intent(IN) :: lm
         END FUNCTION Ylm
      END INTERFACE

      if ( present(Hconv) ) then
         conv=Hconv
      else
         conv=1.d-5
      end if

      if ( all( mask = trho==0.d0 ) ) then
         if (present(iterVh)) iterVh=0
         tVh=0.d0 ; return
      end if

      allocate( zk(ML),tk(ML),pk(ML),wk(-Mx:Mx,-Mx:Mx,-Mx:Mx) )

c------------------------- Boundary condition (multipole expansion)

      rholm=0.d0

      select case( MEO )

c Single center expansion

      case(1)

      a=1

      do L=0,4
      do lm=L**2+1,(L+1)**2
         do i=1,ML
            x=Lx(i)*H ; y=Ly(i)*H ; z=Lz(i)*H
            r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
            rholm(lm,a)=rholm(lm,a)+r**L*Ylm(xx,yy,zz,lm)*trho(i)*H**3
         end do
      end do
      end do

      do i=1,MK
         ix=Kx(i) ; iy=Ky(i) ; iz=Kz(i) ; x=ix*H ; y=iy*H ; z=iz*H
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         sum1=0.d0
         do L=0,4
         do lm=L**2+1,(L+1)**2
            sum1=sum1+Ylm(xx,yy,zz,lm)/r**(L+1)*rholm(lm,a)
         end do
         end do
         wk(ix,iy,iz) = E2*sum1
      end do

c Multi center expansion

      case(2)

      do a=1,MI
      do j=1,Mdv(a)
         i=Ixyz(j,a)
         x=Lx(i)*H-Rion(1,a); y=Ly(i)*H-Rion(2,a); z=Lz(i)*H-Rion(3,a)
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         do L=0,4
         do lm=L**2+1,(L+1)**2
            rholm(lm,a)=rholm(lm,a)+r**L*Ylm(xx,yy,zz,lm)*trho(i)*H**3
         end do
         end do
      end do
      end do

      wk=0.d0

      do a=1,MI
      do i=1,MK
         ix=Kx(i) ; iy=Ky(i) ; iz=Kz(i)
         x=ix*H-Rion(1,a) ; y=iy*H-Rion(2,a) ; z=iz*H-Rion(3,a)
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         sum1=0.d0
         do L=0,4
         do lm=L**2+1,(L+1)**2
            sum1=sum1+Ylm(xx,yy,zz,lm)/r**(L+1)*rholm(lm,a)
         end do
         end do
         wk(ix,iy,iz) = wk(ix,iy,iz) + E2*sum1
      end do
      end do

      end select
c------------------------- C-G minimization

c ini.

      do i=1,ML
         wk(Lx(i),Ly(i),Lz(i))=tVh(i)
      end do

      zk = -4*Pi*E2*trho

      do i=1,ML
         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
         zk(i)=zk(i)-( cN0*3*wk(ix,iy,iz)
     &                +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                      +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                      +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &                +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                      +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                      +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &                +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                      +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                      +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &                +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                      +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                      +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )/(H*H)
      end do

      wk=0.d0 ; pk=zk ; sum1=sum(zk**2)*H**3

      Iteration : do iter=1,maxiter
         do i=1,ML
            wk(Lx(i),Ly(i),Lz(i))=pk(i)
         end do

         do i=1,ML
            ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
            tk(i)=( cN0*3*wk(ix,iy,iz)
     &             +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                   +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                   +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &             +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                   +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                   +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &             +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                   +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                   +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &             +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                   +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                   +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )/(H*H)
         end do

         ak=sum1/sum(zk*tk)/H**3

         tVh=tVh+ak*pk ; zk=zk-ak*tk

         sum2=sum(zk**2)*H**3 ; 
         if ( abs(sum2) < conv ) exit

         ck=sum2/sum1 ; sum1=sum2

         pk=zk+ck*pk
 
      end do Iteration

      if ( present(iterVh) ) iterVh=iter
      if ( iter>maxiter ) then
         write(*,*) "Warning:Vh iteration is not converged, sum2=",sum2
      end if

      deallocate( zk,tk,pk,wk )

      return

      END SUBROUTINE R_Hartree

c=======================================================================
c============================ Hartree potential (Solve Poisson equation)

      SUBROUTINE C_Hartree(trho,tVh,Hconv,iterVh)
      use lda_data
      implicit none
      complex(8),intent(IN)        :: trho(ML)
      complex(8),intent(INOUT)     :: tVh(ML)
      real(8),intent(IN),optional  :: Hconv
      integer,intent(OUT),optional :: iterVh

      integer,parameter :: maxiter=400
      integer :: i,j,ix,iy,iz,lm,L,a,iter
      real(8) :: x,y,z,r,sum1,sum2,ck,xx,yy,zz,conv
      complex(8) :: ak,rholm(25,MI)
      complex(8),allocatable :: zk(:),tk(:),pk(:),wk(:,:,:)
       
      INTERFACE
         FUNCTION Ylm(x,y,z,lm)
         implicit none
         real(8) :: Ylm
         real(8),intent(IN) :: x,y,z
         integer,intent(IN) :: lm
         END FUNCTION Ylm
      END INTERFACE

      if ( present(Hconv) ) then
         conv=Hconv
      else
         conv=1.d-5
      end if

      if ( all( mask = trho==(0.d0,0.d0) ) ) then
         if (present(iterVh)) iterVh=0
         tVh=(0.d0,0.d0) ; return
      end if

      allocate( zk(ML),tk(ML),pk(ML),wk(-Mx:Mx,-Mx:Mx,-Mx:Mx) )

c------------------------- Boundary condition (multipole expansion)

      rholm=(0.d0,0.d0)

      select case( MEO )

c Single center expansion

      case(1)

      a=1

      do L=0,4
      do lm=L**2+1,(L+1)**2
         do i=1,ML
            x=Lx(i)*H ; y=Ly(i)*H ; z=Lz(i)*H
            r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
            rholm(lm,a)=rholm(lm,a)+r**L*Ylm(xx,yy,zz,lm)*trho(i)*H**3
         end do
      end do
      end do

      do i=1,MK
         ix=Kx(i) ; iy=Ky(i) ; iz=Kz(i) ; x=ix*H ; y=iy*H ; z=iz*H
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         sum1=0.d0
         do L=0,4
         do lm=L**2+1,(L+1)**2
            sum1=sum1+Ylm(xx,yy,zz,lm)/r**(L+1)*rholm(lm,a)
         end do
         end do
         wk(ix,iy,iz) = E2*sum1
      end do

c Multi center expansion

      case(2)

      do a=1,MI
      do j=1,Mdv(a)
         i=Ixyz(j,a)
         x=Lx(i)*H-Rion(1,a); y=Ly(i)*H-Rion(2,a); z=Lz(i)*H-Rion(3,a)
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         do L=0,4
         do lm=L**2+1,(L+1)**2
            rholm(lm,a)=rholm(lm,a)+r**L*Ylm(xx,yy,zz,lm)*trho(i)*H**3
         end do
         end do
      end do
      end do

      wk=0.d0

      do a=1,MI
      do i=1,MK
         ix=Kx(i) ; iy=Ky(i) ; iz=Kz(i)
         x=ix*H-Rion(1,a) ; y=iy*H-Rion(2,a) ; z=iz*H-Rion(3,a)
         r=sqrt(x*x+y*y+z*z)+1.d-50 ; xx=x/r ; yy=y/r ; zz=z/r
         sum1=0.d0
         do L=0,4
         do lm=L**2+1,(L+1)**2
            sum1=sum1+Ylm(xx,yy,zz,lm)/r**(L+1)*rholm(lm,a)
         end do
         end do
         wk(ix,iy,iz) = wk(ix,iy,iz) + E2*sum1
      end do
      end do

      end select

c------------------------- C-G minimization

c ini.

      do i=1,ML
         wk(Lx(i),Ly(i),Lz(i))=tVh(i)
      end do

      zk = -4*Pi*E2*trho

      do i=1,ML
         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
         zk(i)=zk(i)-( cN0*3*wk(ix,iy,iz)
     &                +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                      +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                      +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &                +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                      +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                      +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &                +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                      +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                      +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &                +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                      +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                      +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )/(H*H)
      end do

      wk=(0.d0,0.d0) ; pk=zk ; sum1=sum(abs(zk)**2)*H**3

      Iteration : do iter=1,maxiter

         do i=1,ML
            wk(Lx(i),Ly(i),Lz(i))=pk(i)
         end do

         do i=1,ML
            ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
            tk(i)=( cN0*3*wk(ix,iy,iz)
     &             +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                   +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                   +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &             +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                   +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                   +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &             +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                   +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                   +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &             +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                   +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                   +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )/(H*H)
         end do

         ak=sum1/sum(conjg(zk)*tk)/H**3

         tVh=tVh+ak*pk ; zk=zk-ak*tk

         sum2=sum(abs(zk)**2)*H**3 ; if ( abs(sum2) < conv ) exit

         ck=sum2/sum1 ; sum1=sum2

         pk=zk+ck*pk

      end do Iteration

      if ( present(iterVh) ) iterVh=iter
      if ( iter>maxiter ) then
         write(*,*) "Warning:Vh iteration is not converged, sum2=",sum2
      end if

      deallocate( zk,tk,pk,wk )

      return

      END SUBROUTINE C_Hartree
