c=======================================================================
c========================== Hamiltonian Operation ( for real functions )

      SUBROUTINE R_hpsi(tpsi,htpsi)
      use lda_data
      implicit none
      real(8),intent(IN)  :: tpsi(ML)
      real(8),intent(OUT) :: htpsi(ML)

      integer :: ix,iy,iz,i,j,a,lm,ik
      real(8) :: uVpsi
      real(8),allocatable :: wk(:,:,:)

      allocate( wk(-Mx:Mx,-Mx:Mx,-Mx:Mx) ) ; wk=0.d0

      do i=1,ML
         wk(Lx(i),Ly(i),Lz(i))=tpsi(i)
      end do

c Pseudo(local), Hartree, Exchange-Correlation potential

      htpsi = ( Vpsl + Vh + Vxc )*tpsi
 
c Pseudopotential (non-local)

      do a=1,MI
         ik=Kion(a)
         loop_lm : do lm=1,(Mlps(ik)+1)**2
            if ( abs(uVu(lm,a))<1.d-5 ) cycle loop_lm
            uVpsi=0.d0
            do j=1,Mps(a)
               uVpsi=uVpsi+uV(j,lm,a)*tpsi(Jxyz(j,a))
            end do
            uVpsi=uVpsi*H**3/uVu(lm,a)
            do j=1,Mps(a)
               htpsi(Jxyz(j,a))=htpsi(Jxyz(j,a))+uVpsi*uV(j,lm,a)
            end do
         end do loop_lm
      end do

c Kinetic energy

      do i=1,ML
         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
         htpsi(i)=htpsi(i)-H2M/(H*H)*(
     &           +cN0*3*wk(ix,iy,iz)
     &           +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                 +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                 +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &           +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                 +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                 +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &           +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                 +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                 +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &           +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                 +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                 +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )
      end do

      deallocate( wk )

      return

      END SUBROUTINE R_hpsi

c=======================================================================
c========================= Hamiltonian Operation (for complex funcitons)

      SUBROUTINE C_hpsi(tpsi,htpsi)
      use lda_data
      implicit none
      complex(8),intent(IN)  :: tpsi(ML)
      complex(8),intent(OUT) :: htpsi(ML)

      integer :: ix,iy,iz,i,j,a,lm,ik
      complex(8) :: uVpsi
      complex(8),allocatable :: wk(:,:,:)

      allocate( wk(-Mx:Mx,-Mx:Mx,-Mx:Mx) ) ; wk=(0.d0,0.d0)

      do i=1,ML
         wk(Lx(i),Ly(i),Lz(i))=tpsi(i)
      end do

c Pseudo(local), Hartree, Exchange-Correlation potential

      htpsi = ( Vpsl + Vh + Vxc )*tpsi
 
c Pseudopotential (non-local)

      do a=1,MI
         ik=Kion(a)
         loop_lm : do lm=1,(Mlps(ik)+1)**2
            if ( abs(uVu(lm,a))<1.d-5 ) cycle loop_lm
            uVpsi=0.d0
            do j=1,Mps(a)
               uVpsi=uVpsi+uV(j,lm,a)*tpsi(Jxyz(j,a))
            end do
            uVpsi=uVpsi*H**3/uVu(lm,a)
            do j=1,Mps(a)
               htpsi(Jxyz(j,a))=htpsi(Jxyz(j,a))+uVpsi*uV(j,lm,a)
            end do
         end do loop_lm
      end do

c Kinetic energy

      do i=1,ML
         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
         htpsi(i)=htpsi(i)+(-H2M)/(H*H)*(
     &           +cN0*3*wk(ix,iy,iz)
     &           +cN1*( wk(ix+1,iy,iz) + wk(ix-1,iy,iz)
     &                 +wk(ix,iy+1,iz) + wk(ix,iy-1,iz)
     &                 +wk(ix,iy,iz+1) + wk(ix,iy,iz-1) )
     &           +cN2*( wk(ix+2,iy,iz) + wk(ix-2,iy,iz)
     &                 +wk(ix,iy+2,iz) + wk(ix,iy-2,iz)
     &                 +wk(ix,iy,iz+2) + wk(ix,iy,iz-2) )
     &           +cN3*( wk(ix+3,iy,iz) + wk(ix-3,iy,iz)
     &                 +wk(ix,iy+3,iz) + wk(ix,iy-3,iz)
     &                 +wk(ix,iy,iz+3) + wk(ix,iy,iz-3) )
     &           +cN4*( wk(ix+4,iy,iz) + wk(ix-4,iy,iz)
     &                 +wk(ix,iy+4,iz) + wk(ix,iy-4,iz)
     &                 +wk(ix,iy,iz+4) + wk(ix,iy,iz-4) ) )
      end do

      deallocate( wk )

      return

      END SUBROUTINE C_hpsi
