
      MODULE global_variables
      use hyp_data
      implicit none
      integer :: Na,Nb
      END MODULE global_variables

c=======================================================================

      PROGRAM Hypol_Tensor
      use global_variables
      implicit none

      character(20) :: file_IN, process, Hyp_Info
      character(4)  :: XC_in,dir
      integer :: a,b,n
      real(8) :: dshif(3)

c Input

      read(*,*) file_IN
      read(*,*) Hyp_Info
      read(*,*) XC_in
      read(*,*) CG_solver
      read(*,*) process
      read(*,*) dir               ! [ 'x','y','z' or 'xyz' ]
      read(*,*) hw(1),hw(2)
!      read(*,*) CGconv, Hconv, ep

      CGconv=1.d-8
      Hconv =1.d-10
      ep=1.d-12

      select case(dir)
      case('x')   ; Na=1 ; Nb=1
      case('y')   ; Na=2 ; Nb=2
      case('z')   ; Na=3 ; Nb=3
      case('xyz') ; Na=1 ; Nb=3
      end select

c Read LDA data

      open(4,file=file_IN,form='unformatted',status='old')
        call IN_data
      close(4)

c Allocate

      allocate( D(ML,3), dn1(ML,Na:Nb,2) , h1(ML,Na:Nb,2)
     &                 , dn2(ML,Na:Nb,Na:Nb,3), h2(ML,Na:Nb,Na:Nb,3) )
      allocate( Fxc(ML), Gxc(ML), Hxc(ML) )

c Dipole Operator

      dshif(1)=sum(Lx(:ML)*rho)*H/sum(rho) ; D(:,1)=Lx(:ML)*H-dshif(1)
      dshif(2)=sum(Ly(:ML)*rho)*H/sum(rho) ; D(:,2)=Ly(:ML)*H-dshif(2)
      dshif(3)=sum(Lz(:ML)*rho)*H/sum(rho) ; D(:,3)=Lz(:ML)*H-dshif(3)

c Exchange-Correlation Kernel

      call Exc_Cor_Kernel(XC_in,Fxc,Gxc,Hxc)

c Initialize

      dn1=0.d0     ; h1=dn1       ; dn2=0.d0       ; h2=dn2
      alphaF=0.d0  ; alpha=alphaF ; alphaF_av=0.d0 ; alpha_av=alphaF_av
      betaF=0.d0   ; beta=betaF   ; betaF_av=0.d0  ; beta_av=betaF_av
      gammaF=0.d0  ; gamma=gammaF ; gammaF_av=0.d0 ; gamma_av=gammaF_av

c Timer On

      call watch

c

      write(*,'(/1x,"hw1, hw2      =",3f10.5)') hw(1),hw(2)
      write(*,*)    "process       = ", process
      write(*,*)    "dir           = ", dir
      write(*,*)    "lda file      = ", file_IN
      write(*,*)    "xc kernel     = ", XC_in
      write(*,'(1x,"CRconv, CGconv, Hconv =",3g12.4)') ep, CGconv, Hconv
      write(*,'(1x,"dshif =",3g14.4/)') dshif(1), dshif(2), dshif(3)


      select case(process)

c---------------------------------------------- EFISH

      case('EFISH','efish')

c alpha

         const=-E2

         n=1
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

         n=2
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

c beta(-2w;w,w)

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c beta(-w;0,w), beta(-w;w,0)

         n=2
         do a=Na,Nb
         do b=Na,Nb
            call calc_beta( n, a,b, 2,1, hw(2),hw(1) )
              dn2(:,b,a,n+1) =   dn2(:,a,b,n)
               h2(:,b,a,n+1) =    h2(:,a,b,n)
            betaF(:,b,a,n+1) = betaF(:,a,b,n)
             beta(:,b,a,n+1) =  beta(:,a,b,n)
            call watch(DISP='on')
         end do
         end do

! gamma_aabb(-2w;0,w,w)

         const = -E2**2

         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma( a,b,b, 2,1,1, 1,3,2, hw(2),hw(1),hw(1) )
            call watch(DISP='on')
         end do
         end do

! gamma_abba(-2w;0,w,w), gamma_abab(-2w;0,w,w)

         do a=Na,Nb
         do b=Na,Nb
            if ( a==b ) cycle
            call calc_gamma( b,b,a, 2,1,1, 1,3,2, hw(2),hw(1),hw(1) )
            gammaF(:,b,a,b)=gammaF(:,b,b,a)
            gamma(:,b,a,b)=gamma(:,b,b,a)
            call watch(DISP='on')
         end do
         end do

c---------------------------------------------- DFWM

      case('DFWM','dfwm','IDRI','idri')

         const=-E2

         n=1
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

         n=2
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) ) 
            call watch(DISP='on')
         end do

c beta(-2w;w,w) ( SHG )

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c beta(0;w,-w), beta(0;-w,w) ( Optical Rectification )

         n=2
         do a=Na,Nb
         do b=Na,Nb
            call calc_beta( n, a,b, 1,2, hw(1),hw(2) )
              dn2(:,b,a,n+1) =   dn2(:,a,b,n)
               h2(:,b,a,n+1) =    h2(:,a,b,n)
            betaF(:,b,a,n+1) = betaF(:,a,b,n)
             beta(:,b,a,n+1) =  beta(:,a,b,n)
            call watch(DISP='on')
         end do
         end do

c gamma_aabb(-w;w,-w,w), gamma_abba(-w;w,-w,w)

         const = -E2**2

         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma( a,b,b, 1,2,1, 3,1,2, hw(1),hw(2),hw(1) )
            gammaF(:,b,b,a)=gammaF(:,a,b,b)
            gamma(:,b,b,a)=gamma(:,a,b,b)
            call watch(DISP='on')
         end do
         end do

c gamma_abab(-w;w,-w,w)

         do a=Na,Nb
         do b=Na,Nb
            if ( a==b ) cycle
            call calc_gamma( b,a,b, 1,2,1, 3,1,2, hw(1),hw(2),hw(1) )
            call watch(DISP='on')
         end do
         end do

c----------------------------------------------- THG

      case default

         const=-E2

         n=1
         do a=Na,Nb
            call calc_alpha( n, a, hw(n) )
            call watch(DISP='on')
         end do

c beta(-2w;w,w)

         const = -E2**1.5d0

         n=1
         do a=Na,Nb
            do b=Na,a
               call calc_beta( n, a,b, 1,1, hw(1),hw(1) )
                 dn2(:,b,a,n) =   dn2(:,a,b,n)
                  h2(:,b,a,n) =    h2(:,a,b,n)
               betaF(:,b,a,n) = betaF(:,a,b,n)
                beta(:,b,a,n) =  beta(:,a,b,n)
               call watch(DISP='on')
            end do
         end do

c gamma(-3w;w,w,w)

         const = -E2**2

         n=1
         do a=Na,Nb
         do b=Na,Nb
            call calc_gamma( a,b,b, 1,1,1, 1,1,1, hw(1),hw(1),hw(1) )
            gammaF(:,b,b,a)=gammaF(:,a,b,b)
            gamma(:,b,b,a) =gamma(:,a,b,b)
            gammaF(:,b,a,b)=gammaF(:,a,b,b)
            gamma(:,b,a,b) =gamma(:,a,b,b)
            call watch(DISP='on')
         end do
         end do

      end select

c

      open(1,file=Hyp_INFO)
         call write_results
      close(1)

      stop

      END PROGRAM Hypol_Tensor

c=============================================================== Results

      SUBROUTINE write_results
      use global_variables
      implicit none

      real(8),parameter :: B2esu = 790.034d0 , G2esu = 0.624153d0
      real(8),parameter :: Aau2esu = 0.148183d0, Bau2esu = 8.63905d0
      real(8),parameter :: Gau2esu = 5.03654d-4
      integer :: a,b,n
      real(8) :: U


      write(1,'(1x,"hw(1) =",f9.5,"      hw(2) =",f9.5)') hw(1),hw(2)

c alpha

      do n=1,2
         write(1,'(/1x,"alpha(",i1,")  (A^3)")') n
         write(1,'(1x,3(:"(",2i1,")",f15.7,4x))')
     &        ( a,a, alpha(a,a,n) ,a=Na,Nb )
      end do

      write(1,*)

c beta

      U=B2esu/Bau2esu           !( a.u. )

      do n=1,2
         b=Nb
         write(1,'(/1x,"beta(",i1,")  (a.u.)")') n
         write(1,'(1x,"(",3i1,")",f15.7)') b,b,b,beta(b,b,b,n)*U
         do a=Na,b-1
            write(1,'(3(1x,"(",3i1,")",f15.7,4x))')
     &           b,a,a,beta(b,a,a,n)*U
     &           ,a,b,a,beta(a,b,a,n)*U
     &           ,a,a,b,beta(a,a,b,n)*U
         end do
      end do

      write(1,*)

c gamma

      U=G2esu/Gau2esu           !( a.u. )

      write(1,'(/1x,"gamma  (a.u.)")')

      write(1,'(1x,3(:"(",4i1,")",f15.7,4x))')
     &     ( a,a,a,a,gamma(a,a,a,a)*U, a=Na,Nb )

      do a=Na,Nb
         do b=a+1,Nb
            write(1,'(1x,3("(",4i1,")",f15.7,4x))')
     &           a,a,b,b,gamma(a,a,b,b)*U
     &           ,a,b,b,a,gamma(a,b,b,a)*U
     &           ,a,b,a,b,gamma(a,b,a,b)*U
         end do
      end do

      write(1,'(/)')

c average

c alpha

      alpha_av=0.d0 ; alphaF_av=0.d0
      do a=Na,Nb
         alpha_av = alpha_av + alpha(a,a,:)
         alphaF_av = alphaF_av + alphaF(a,a,:)
      end do
      if(Na/=Nb)then
         alpha_av=alpha_av/3.d0 ; alphaF_av=alphaF_av/3.d0
      end if

      do n=1,2
         write(1,'(1x,"       <alpha(",i1,")>"
     &               ,"            <alpha_free(",i1,")>")') n,n
         write(1,'(1x,2g24.15)') alpha_av(n),alphaF_av(n)
         write(1,'(1x,2g24.15,"  (a.u.)"/)') alpha_av(n)/Aau2esu
     &        ,alphaF_av(n)/Aau2esu
      end do

c beta

      beta_av=0.d0 ; betaF_av=0.d0 ; b=Nb
      do a=Na,Nb
         beta_av=beta_av+beta(b,a,a,:)+beta(a,b,a,:)+beta(a,a,b,:)
         betaF_av=betaF_av+betaF(b,a,a,:)
     &        +betaF(a,b,a,:)+betaF(a,a,b,:)
      end do
      if(Na/=Nb)then
         beta_av=beta_av/5.d0 ; betaF_av=betaF_av/5.d0
      else
         beta_av=beta_av/3.d0 ; betaF_av=betaF_av/3.d0
      end if

      write(*,*)

      do n=1,2
         write(1,'(1x,"        <beta(",i1,")>"
     &                ,"            <beta_free(",i1,")>")') n,n
         write(1,'(1x,2g24.15)') beta_av(n),betaF_av(n)
         write(1,'(1x,2g24.15,"  (esu)")') beta_av(n)*B2esu
     &        ,betaF_av(n)*B2esu
         write(1,'(1x,2g24.15,"  (a.u.)"/)') beta_av(n)*B2esu/Bau2esu
     &        ,betaF_av(n)*B2esu/Bau2esu
      end do

      write(*,*)

c gamma

      gamma_av=0.d0 ; gammaF_av=0.d0
      do a=Na,Nb
         do b=Na,Nb
            gamma_av=gamma_av
     &           + gamma(a,a,b,b)+gamma(a,b,b,a)+gamma(a,b,a,b)
            gammaF_av=gammaF_av
     &           + gammaF(a,a,b,b)+gammaF(a,b,b,a)+gammaF(a,b,a,b)
         end do
      end do
      if(Na/=Nb)then
         gamma_av=gamma_av/15.d0 ; gammaF_av=gammaF_av/15.d0
      else
         gamma_av=gamma_av/3.d0  ; gammaF_av=gammaF_av/3.d0
      end if

      write(1,'(1x,"         <gamma>"
     &            ,"              <gamma_free>")')
      write(1,'(1x,2g24.15)') gamma_av,gammaF_av
      write(1,'(1x,2g24.15,"  (esu)")') gamma_av*G2esu,gammaF_av*G2esu
      write(1,'(1x,2g24.15,"  (a.u.)"/)') gamma_av*G2esu/Gau2esu
     &     ,gammaF_av*G2esu/Gau2esu

      return

      END SUBROUTINE write_results
