!=======================================================================
!========================================= Grid generation and labelling

      SUBROUTINE init_mesh
      use lda_data
      implicit none

      integer,allocatable :: Lxyz(:,:,:)
      integer :: ix,iy,iz,i,a,il,ik,amin
      real(8) :: r,z,rmin
      real(8),parameter :: epsilon=1.d-10

      print *,"----------------------------------- init_mesh"

      select case( Box_shape )
         case(1,3) ; Mx=int(Rsize/H)+Nd
         case(2)   ; Mx=max( int(Rsize/H)+Nd, int(Zsize/H)+Nd )
      end select

      write(*,*) "Mx     =", Mx

      allocate( Lxyz(-Mx:Mx,-Mx:Mx,-Mx:Mx) ) ; Lxyz=0

!-------------------- Division of area to inner(=1) and outer(=2) one

      select case( Box_shape )

! Spherical Box Case

      case(1)

         do ix=-Mx+Nd,Mx-Nd
         do iy=-Mx+Nd,Mx-Nd
         do iz=-Mx+Nd,Mx-Nd
            r=sqrt((ix*ix+iy*iy+iz*iz)*H*H)
            if ( r-Rsize <= epsilon ) then
               do i=-Nd,Nd
                  Lxyz(ix+i,iy,iz) = 2
                  Lxyz(ix,iy+i,iz) = 2
                  Lxyz(ix,iy,iz+i) = 2
               end do
            end if
         end do
         end do
         end do

         do ix=-Mx+Nd,Mx-Nd
         do iy=-Mx+Nd,Mx-Nd
         do iz=-Mx+Nd,Mx-Nd
            r=sqrt((ix*ix+iy*iy+iz*iz)*H*H)
            if ( r-Rsize <= epsilon ) Lxyz(ix,iy,iz) = 1
         end do
         end do
         end do

! Cylindrical Box Case

      case(2)

      do ix=-Mx+Nd,Mx-Nd
      do iy=-Mx+Nd,Mx-Nd
      do iz=-Mx+Nd,Mx-Nd
         r=sqrt((ix*ix+iy*iy)*H*H) ; z=iz*H
         if ( r-Rsize<=epsilon .and. abs(z)-Zsize<=epsilon ) then
            do i=-Nd,Nd
               Lxyz(ix+i,iy,iz) = 2
               Lxyz(ix,iy+i,iz) = 2
               Lxyz(ix,iy,iz+i) = 2
            end do
         end if
      end do
      end do
      end do

      do ix=-Mx+Nd,Mx-Nd
      do iy=-Mx+Nd,Mx-Nd
      do iz=-Mx+Nd,Mx-Nd
         r=sqrt((ix*ix+iy*iy)*H*H) ; z=iz*H
         if ( r-Rsize<=epsilon .and. abs(z)-Zsize<=epsilon )
     &      Lxyz(ix,iy,iz) = 1
      end do
      end do
      end do

! Minimal Box

      case(3)

      do a=1,MI
         do ix=-Mx+Nd,Mx-Nd
         do iy=-Mx+Nd,Mx-Nd
         do iz=-Mx+Nd,Mx-Nd
            r=sqrt( (ix*H-Rion(1,a))**2+(iy*H-Rion(2,a))**2
     &             +(iz*H-Rion(3,a))**2 )
            if ( r-Rsize<=epsilon ) then
               do i=-Nd,Nd
                  Lxyz(ix+i,iy,iz) = 2
                  Lxyz(ix,iy+i,iz) = 2
                  Lxyz(ix,iy,iz+i) = 2
               end do
            end if
         end do
         end do
         end do
      end do

      do a=1,MI
         do ix=-Mx+Nd,Mx-Nd
         do iy=-Mx+Nd,Mx-Nd
         do iz=-Mx+Nd,Mx-Nd
            r=sqrt((ix*H-Rion(1,a))**2+(iy*H-Rion(2,a))**2
     &            +(iz*H-Rion(3,a))**2)
            if ( r-Rsize<=epsilon ) Lxyz(ix,iy,iz) = 1
         end do
         end do
         end do
      end do

      end select

!---------- Counting the number of inner and outer grid points

      il=0 ; ik=0
      do ix=-Mx,Mx
      do iy=-Mx,Mx
      do iz=-Mx,Mx
         if ( Lxyz(ix,iy,iz)==1 ) il=il+1
         if ( Lxyz(ix,iy,iz)==2 ) ik=ik+1
      end do
      end do
      end do

      ML=il ; MK=ik

      write(*,*) "ML, NL =", ML,NL
      write(*,*) "MK, NK =", MK,NK

      if ( ML>NL .or. MK>NK ) then
         write(*,*) "NL,NK should be larger than ML,MK."
         stop
      end if

      il=0 ; ik=0
      do ix=-Mx,Mx
      do iy=-Mx,Mx
      do iz=-Mx,Mx
         if ( Lxyz(ix,iy,iz)==1 ) then
            il=il+1 ; Lx(il)=ix; Ly(il)=iy; Lz(il)=iz
         end if
         if ( Lxyz(ix,iy,iz)==2 ) then
            ik=ik+1 ; Kx(ik)=ix; Ky(ik)=iy; Kz(ik)=iz
         end if
      end do
      end do
      end do

      deallocate( Lxyz )

!-------------------- Division of Area for Multipole Expansion

      if ( MEO /= 2 ) return

      Mdv=0 ; Ixyz=0

      do i=1,ML
         rmin=1.d6
         do a=1,MI
            r=sqrt( (Lx(i)*H-Rion(1,a))**2
     &             +(Ly(i)*H-Rion(2,a))**2
     &             +(Lz(i)*H-Rion(3,a))**2 )
            if ( r < rmin ) then
               rmin=r ; amin=a
            end if
         end do
         Mdv(amin)=Mdv(amin)+1
      end do

      do a=1,MI
         if ( Mdv(a) > Ndv ) then
            amin=0
            write(*,*) "Ndv should be larger than ", Mdv(a)
         end if
      end do
      if ( amin==0 ) stop

      maxMdv=maxval(Mdv)

      write(*,*) "max( Mdv(a) ) = ",maxMdv

      Mdv=0
      do i=1,ML
         rmin=1.d6
         do a=1,MI
            r=sqrt( (Lx(i)*H-Rion(1,a))**2
     &             +(Ly(i)*H-Rion(2,a))**2
     &             +(Lz(i)*H-Rion(3,a))**2 )
            if ( r < rmin ) then
               rmin=r ; amin=a
            end if
         end do
         Mdv(amin)=Mdv(amin)+1
         Ixyz(Mdv(amin),amin)=i
      end do

      return

      END SUBROUTINE init_mesh
