!=======================================================================
!======================================================= Pseudopotential

      SUBROUTINE init_ps
      use lda_data
      implicit none

      integer,parameter :: Nr=5000, Nlps=3, Nlmps=16

      integer :: i,L,ik,Mlps0
      real(8) :: upp(0:Nr,0:Nlps),r,rPC,rRC(0:Nlps),Rps0
      integer :: j,a,intr,lm
      real(8) :: x,y,z,ratio1,ratio2,sum0,url,uVrl

      integer :: Mr(MKI)
      real(8) :: step(MKI)
      real(8) :: rhopp(0:Nr,MKI),rho_core(ML)
      real(8) :: vpp(0:Nr,0:Nlps,MKI)
      real(8) :: uppr(0:Nr,0:Nlps,MKI)
      real(8) :: u(Nps,Nlmps),uVnl(0:Nr,0:Nlps,MKI)

      INTERFACE
         FUNCTION Ylm(x,y,z,lm)
         implicit none
         real(8) :: Ylm
         real(8),intent(IN) :: x,y,z
         integer,intent(IN) :: lm
         END FUNCTION Ylm
      END INTERFACE

      print *,"----------------------------------- init_ps"

!-------------------- Read atomic pseudopotential data

      element : do ik=1,MKI

         select case( Zatom(ik) )

         case(1) ; open(4,file='H_rps.dat'  ,status='old')
         case(2) ; open(4,file='Li_rps.dat' ,status='old')
         case(6) ; open(4,file='C_rps.dat'  ,status='old')
         case(7) ; open(4,file='N_rps.dat'  ,status='old')
         case(8) ; open(4,file='O_rps.dat'  ,status='old')
         case(10); open(4,file='Ne_crps.dat',status='old')
         case(11); open(4,file='Na_rps.dat' ,status='old')
         case(12); open(4,file='Mg_rps.dat' ,status='old')
         case(13); open(4,file='Al_rps.dat' ,status='old')
         case(18); open(4,file='Ar_crps.dat',status='old')
         case(29); open(4,file='Cu_rps.dat' ,status='old')
         case(36); open(4,file='Kr_crps.dat',status='old')

         end select

         read(4,*) Mr(ik),step(ik),Mlps0,Zps(ik)

         if ( Mlps0 < Mlps(ik) ) then
            write(*,*) "too large Mlps value for ",ik," atom"
            stop
         end if

         read(4,*) rPC, ( rRC(L) , L=0,Mlps0 )

         Rps(ik)=maxval( rRC )

         print *,ik,'Rps=',Rps(ik)

         do i=0,Mr(ik)
            read(4,*) r,rhopp(i,ik),(vpp(i,L,ik),L=0,Mlps0)
         end do
         do i=0,Mr(ik)
            read(4,*) r,(upp(i,L),L=0,Mlps0)
         end do

         close(4)

         do L=0,Mlps0
            do i=1,Mr(ik)
               r=i*step(ik)
               uppr(i,L,ik)=upp(i,L)/r**(L+1)*sqrt((2*L+1)/(4*Pi))
            end do
            uppr(0,L,ik)=uppr(1,L,ik)
         end do

         rhopp(Mr(ik)+1:Nr,ik)=0.d0 ; uppr(Mr(ik)+1:Nr,:,ik)=0.d0

         do L=0,Mlps0
            do i=Mr(ik)+1,Nr
               r=i*step(ik) ; vpp(i,L,ik)=-Zps(ik)*E2/r
            end do
         end do

      end do element

!-------------------- Pseudopotential for 3-D calculation

      Mps=0

      do ik=1,MKI
      do L=0,Mlps(ik)
      do i=0,Nr
         uVnl(i,L,ik)=(vpp(i,L,ik)-vpp(i,Lref(ik),ik))*uppr(i,L,ik)
      end do
      end do
      end do

      do a=1,MI
         ik=Kion(a) ; j=0
         do i=1,ML 
            r=sqrt( (Lx(i)*H-Rion(1,a))**2
     &             +(Ly(i)*H-Rion(2,a))**2
     &             +(Lz(i)*H-Rion(3,a))**2 )
            if ( r < Rps(ik) ) j=j+1
         end do
         Mps(a)=j ; write(*,*) "Mps, Nps =", Mps(a),Nps
      end do

      maxMps=maxval(Mps)

      write(*,*) "max( Mps(a) ) = ",maxMps

      if ( maxMps>Nps ) then
         write(*,*) "Nps should be larger than", maxMps
         stop
      end if

      do a=1,MI
         ik=Kion(a) ; j=0
         do i=1,ML 
            r=sqrt((Lx(i)*H-Rion(1,a))**2+(Ly(i)*H-Rion(2,a))**2
     &            +(Lz(i)*H-Rion(3,a))**2)
            if ( r < Rps(ik) ) then
               j=j+1 ; Jxyz(j,a)=i
             end if
         end do
      end do

      Mlmps=(maxval(Mlps)+1)**2 ; write(*,*) "Mlmps =",Mlmps

      allocate( uV(maxMps,Mlmps,MI), uVu(Mlmps,MI) )

      Ion : do a=1,MI

         ik=Kion(a)

         do L=0,Mlps(ik)
         do j=1,Mps(a)
            x=Lx(Jxyz(j,a))*H-Rion(1,a)
            y=Ly(Jxyz(j,a))*H-Rion(2,a)
            z=Lz(Jxyz(j,a))*H-Rion(3,a)
            r=sqrt(x**2+y**2+z**2)+1.d-50
            intr=int(r/step(ik))
            ratio1=(r/step(ik)-intr) ; ratio2=1.d0-ratio1
            url =ratio1*uppr(intr+1,L,ik)+ratio2*uppr(intr,L,ik)
            uVrl=ratio1*uVnl(intr+1,L,ik)+ratio2*uVnl(intr,L,ik)
            do lm=L**2+1,(L+1)**2
               u(j,lm)=url*Ylm(x,y,z,lm) ; uV(j,lm,a)=uVrl*Ylm(x,y,z,lm)
            end do
         end do
         end do

         do lm=1,(Mlps(ik)+1)**2
            uVu(lm,a)=sum(uV(:Mps(a),lm,a)*u(:Mps(a),lm))*H**3
            if ( abs(uVu(lm,a)) < 1.d-10 ) uVu(lm,a)=1.d-10
         end do
         write(*,'(1x,"ion = ",i3,"  uVu integral table")') a
         write(*,'(1x,4f15.5)') (uVu(lm,a),lm=1,(Mlps(ik)+1)**2)
 
      end do Ion
 
      Vpsl=0.d0 ; rho_core=0.d0

      do a=1,MI
         ik=Kion(a)
         do i=1,ML
            r=sqrt( (Lx(i)*H-Rion(1,a))**2
     &             +(Ly(i)*H-Rion(2,a))**2
     &             +(Lz(i)*H-Rion(3,a))**2 )+1.d-50
            intr=int(r/step(ik))
            ratio1=r/step(ik)-intr ; ratio2=1.d0-ratio1
            Vpsl(i)=Vpsl(i)+ratio1*vpp(intr+1,Lref(ik),ik)
     &                     +ratio2*vpp(intr,Lref(ik),ik)
            rho_core(i)=rho_core(i)+ratio1*rhopp(intr+1,ik)
     &                       +ratio2*rhopp(intr,ik)
         end do
      end do
 
      return

      END SUBROUTINE init_ps
