************************************************************************
*                                                                      *
* Real-space, real-time TDDFT program                                  *
* prepared 9 Aug 2001                                                  *
* by J.-I.Iwata and K.Yabana, University of Tsukuba                    *
*                                                                      *
************************************************************************

      MODULE lda_data

      implicit none

!-------------------- Parameters

      integer,parameter :: NI=60      ! Maximum number of atoms

! Grid points
      integer,parameter :: NL=100000  ! Number of (inner) grid points
      integer,parameter :: NK=100000  ! Number of (outer) grid points
      integer,parameter :: Ndv=100000 ! Number of grid points around an ion
      integer,parameter :: Nps=1000   ! Number of grid points inside of the
                                      ! radius of a pseudopotential
! Physical constants
      real(8),parameter :: E2=14.4d0, H2M=3.81d0, a_B=0.529177d0
      real(8),parameter :: Ry=13.6058d0, Pi=3.141592653589793d0

! Coefficients of finite-difference
      integer,parameter :: Nd=4      ! (2*Nd+1)-points finite-difference
      real(8),parameter :: cN0=-205.d0/72.d0                 ! for Laplacian
      real(8),parameter :: cN1=8.d0/5.d0  , cN2=-1.d0/5.d0
      real(8),parameter :: cN3=8.d0/315.d0, cN4=-1.d0/560.d0
      real(8),parameter :: bN1=4.d0/5.d0  , bN2=-1.d0/5.d0   ! for Nabla
      real(8),parameter :: bN3=4.d0/105.d0, bN4=-1.d0/280.d0

!-------------------- Global variables

      integer :: Miter       ! Total number of Iteration for LDA

      integer :: MI,MKI,MST,Mx

      integer,allocatable :: Kion(:)    ! Label for ions
      real(8),allocatable :: Rion(:,:)  ! Ion cordinates
      integer,allocatable :: Zatom(:)   ! Charge of ion

      integer :: Box_shape   ! Box shape
      real(8) :: H           ! Grid spacing
      real(8) :: Rsize,Zsize ! Radius and height of the box

! Labels of grid points
      integer :: ML,MK
      integer :: Lx(NL),Ly(NL),Lz(NL)
      integer :: Kx(NK),Ky(NK),Kz(NK)
      integer :: Ixyz(Ndv,NI),Mdv(NI),maxMdv,MEO
      integer :: Jxyz(Nps,NI),Mps(NI),maxMps

! Pseudopotential
      integer :: Mlmps
      integer,allocatable :: Mlps(:),Lref(:)
      real(8),allocatable :: Zps(:)              ! Pseudo charge
      real(8),allocatable :: Rps(:)              ! Core radius
      real(8),allocatable :: Vpsl(:)             ! Local pseudopotential
      real(8),allocatable :: uV(:,:,:),uVu(:,:)  ! Non-local

      real(8),allocatable :: occ(:)      ! Occupation number
      real(8),allocatable :: psi(:,:)    ! Single particle orbitals
      real(8),allocatable :: esp(:)      ! Single particle energy
      real(8),allocatable :: rho(:)      ! Single particle density
      real(8),allocatable :: Vh(:)       ! Hartree potential
      real(8),allocatable :: Vxc(:)      ! Exchange-Correlation potential

! Exchange-Correlation potential
      character(4) :: XC

      CONTAINS

!=======================================================================

      SUBROUTINE OUT_data

      write(1) NL,NK,NI,Ndv,Nps,Nd
      write(1) Mx,ML,MK,MST,MI,MKI,maxMdv,maxMps,Mlmps
      write(1) H
      write(1) Rsize,Zsize,Box_shape
      write(1) Miter
      write(1) Lx(:ML),Ly(:ML),Lz(:ML),Kx(:ML),Ky(:ML),Kz(:ML)
      write(1) Ixyz(:maxMdv,:MI),Mdv(:MI),MEO
      write(1) Jxyz(:maxMps,:MI),Mps(:MI)
      write(1) Kion(:MI)
      write(1) Rion(:,:MI)
      write(1) Zatom(:MKI)
      write(1) Zps(:MKI)
      write(1) psi(:ML,:MST),rho(:ML)
      write(1) esp(:MST),occ(:MST)
      write(1) Vh(:ML),Vxc(:ML),Vpsl(:ML)
      write(1) uV(:maxMps,:Mlmps,:MI),uVu(:Mlmps,:MI)
      write(1) Mlps(:MKI),Lref(:MKI)
      write(1) XC
      
      END SUBROUTINE OUT_data

!=======================================================================

      SUBROUTINE IN_data

      integer :: NL0,NK0,NI0,Ndv0,Nps0,Nd0

      read(4) NL0,NK0,NI0,Ndv0,Nps0,Nd0

      if (  NL0/=NL   .or. NK0/=NK   .or. NI0/=NI
     & .or. Ndv0/=Ndv .or. Nps0/=Nps .or. Nd0/=Nd ) then
         write(*,*) "Nx,NL,NK,NST,NI,NKI"
         write(*,*) "Ndv,Nps,Nd ="
         write(*,*) "  present parameters"
         write(*,*) NL,NK,NI,Ndv,Nps,Nd
         write(*,*) "  previous parameters"
         write(*,*) NL0,NK0,NI0,Ndv0,Nps0,Nd0
         stop
      end if

      read(4) Mx,ML,MK,MST,MI,MKI,maxMdv,maxMps,Mlmps
      read(4) H
      read(4) Rsize,Zsize,Box_shape
      read(4) Miter
      read(4) Lx(:ML),Ly(:ML),Lz(:ML),Kx(:ML),Ky(:ML),Kz(:ML)
      read(4) Ixyz(:maxMdv,:MI),Mdv(:MI),MEO
      read(4) Jxyz(:maxMps,:MI),Mps(:MI)

      allocate( Kion(MI),Rion(3,MI),Zatom(MKI) )
      allocate( Zps(MKI) )
      allocate( psi(ML,MST),rho(ML),esp(MST),occ(MST) )
      allocate( Vh(ML),Vxc(ML),Vpsl(ML) )
      allocate( uV(maxMps,Mlmps,MI),uVu(Mlmps,MI) )
      allocate( Mlps(MKI),Lref(MKI) )

      read(4) Kion(:MI)
      read(4) Rion(:,:MI)
      read(4) Zatom(:MKI)
      read(4) Zps(:MKI)
      read(4) psi(:ML,:MST),rho(:ML)
      read(4) esp(:MST),occ(:MST)
      read(4) Vh(:ML),Vxc(:ML),Vpsl(:ML)
      read(4) uV(:maxMps,:Mlmps,:MI),uVu(:Mlmps,:MI)
      read(4) Mlps(:MKI),Lref(:MKI)

      write(*,'(/,1x,"------------------------------ LDA parameters")')
      write(*,'(1x,"MI        =",i4)') MI
      write(*,'(1x,"MKI:Zatom =",:4(2i4,3x))')(NL0,Zatom(NL0),NL0=1,MKI)
      write(*,'(1x,"MST       =",i4)') MST
      write(*,'(1x,"Mx,ML,MK  =",i4,2i8)') Mx,ML,MK
      write(*,'(1x,"Box_shape,Rsize,Zsize,H =",i4,3f10.5)')
     &                                Box_shape,Rsize,Zsize,H

      read(4,END=999) XC
      write(*,'(1x,"XC        =",a5)') XC

 999  write(*,*)

      END SUBROUTINE IN_data

      END MODULE lda_data
