c(Input parameters format)***********************************************
c
c  IC, OC                              [ integer, integer          ]
c  Diter, Ncg                          [ integer, integer          ]
c  file_IN                             [ character(30)             ]
c  file_OUT                            [ character(30)             ]
c  LDA_Info                            [ character(30)             ]
c  XC                                  [ character(4)              ]
c
c  MI,MKI                              [ integer, integer          ]
c  Zatom(1), ... ,Zatom(MKI)           [ integer, ...              ]
c
c  a, Rion(1,a), Rion(2,a), Rion(3,a), Kion(a)   [ integer, real(8), real(8)
c  .    .          .          .          .                , real(8), integer ]
c  .    .          .          .          .
c  .    .          .          .          .
c
c  Box_shape                           [ integer                   ]
c  H, Rsize, Zsize                     [ real(8), real(8), real(8) ]
c  MEO                                 [ integer                   ]
c  MST, MST_HOMO                       [ integer, integer          ]
c  Mlps(1), ... ,Mlps(MKI)             [ integer, ...              ]
c  Lref(1), ... ,Lref(MKI)             [ integer, ...              ]
c
c(Explanation)***********************************************************
c
c  IC    = 0     ! 0: new calculation,  1: continue (read data from file_IN)
c  OC    = 1     ! 1: Write final data to 'file_OUT'
c  Diter = 100   ! Number of iteration for present calculation
c  Ncg   = 4     ! Iteration number in 'DTcg' (1-4)
c
c  file_IN  = 'C2H4-R5H3.data'  ! lda data (w.f., den., etc.) save file name
c  file_OUT = file_IN           ! previous lda data read file name
c  LDA_Info = 'C2H4-R5H3.info'  ! Write present information (eigenvalues, etc.)
c  XC       = 'PZ'              ! Exchange-Correlation [PZ,VWN,LB94]
c
c  Box_shape = 1    ! Box_shape
c                   !  1 : Spherical box with Rsize radius
c                   !  2 : Cylindrical Box of Rsize,Zsize
c                   !  3 : Minimal box with Rsize atom size
c
c  H     = 0.3d0    ! Grid-spacing (in angstrom)
c  Rsize = 5.d0     ! Radius (in angstrom)
c  Zsize = 0.d0     ! Height of the Cylinder ( Box_shape=2 )
c
c  MEO=2            ! MEO=1 : Single center expansion
c                   !    =2 : Multi center expansion
c
c  MST      = 20    ! Total number of orbitals (occ.+virt.)
c  MST_HOMO = 6     ! Number of occupied orbitals
c
c  Mlps(1:MKI) = 1  ! Maximum angular momentum component
c                   ! of the pseudopotential
c
c  Lref(1:MKI) = 1  ! Angular momentum of
c                   ! reference (local) pseudopotential
c
!=======================================================================

      PROGRAM Real_Space_DFT
      use lda_data
      use lda_interfaces
      implicit none

      character(30) :: file_OUT,file_IN,LDA_Info
      character(4)  :: XC_in
      integer       :: IC,OC,a,ik,p,iter,iterVh,Ncg,Diter
      integer       :: MST_HOMO
      real(8)       :: Etot,Exc,Hconv

      read(*,*) IC,OC
      read(*,*) Diter, Ncg
      read(*,*) file_IN
      read(*,*) file_OUT
      read(*,*) LDA_Info
      read(*,*) XC_in

c 'Hartree' parameter

      Hconv  = 1.d-5     ! Convergence criterion
      iterVh = 0         ! Iteration counter


      select case( IC )


c----- New calculation -----

      case default

         Miter = 0       ! Miter: Iteration counter set to zero

         XC = XC_in

c Ions data

         read(*,*) MI,MKI

         print *,"----------------------------------- init_ion"
         write(*,*) "MI,NI =",MI,NI
         if ( MI > NI ) stop " NI is smaller than MI."

         allocate( Zatom(MKI),Rion(3,MI),Kion(MI) )

         read(*,*) ( Zatom(ik) , ik=1,MKI )

         do a=1,MI
            read(*,*) ik,Rion(1,a),Rion(2,a),Rion(3,a),Kion(a)
         end do

         allocate( Mlps(MKI),Lref(MKI) )

         read(*,*) Box_shape
         read(*,*) H, Rsize, Zsize
         read(*,*) MEO
         read(*,*) MST,MST_HOMO
         read(*,*) ( Mlps(ik) , ik=1,MKI )
         read(*,*) ( Lref(ik) , ik=1,MKI )

!         allocate( occ(MST) ) ; occ=2.d0        ! Occupation number
         allocate( occ(MST) ) ; occ=0.d0 ; occ(:MST_HOMO)=2.d0

         call init_mesh
         allocate( Vpsl(ML),Zps(MKI),Rps(MKI) )
         call init_ps
         allocate( psi(ML,MST) ) ; psi=0.d0
         call init_wf(psi)
         call Gram_Schmidt
         allocate( rho(ML) ) ; rho=0.d0
         call psi_rho(0.d0,1.d0)
         allocate( Vh(ML) ) ; Vh=0.d0
         call Hartree(rho,Vh,Hconv,iterVh)
         allocate( Vxc(ML) )
         call Exc_cor(XC,Exc)
         allocate( esp(MST) )
         call Total_Energy

c----- Continue the previous calculation -----

      case(1)

         open(4,file=file_IN,form='unformatted',status='old')
            call IN_data
         close(4)

         XC=XC_in

      end select

      write(*,*) 'present XC = ',XC

c Timer on

      call watch

!------------------------------------------------------------- Iteration

      Iteration : do iter=1,Diter

         Miter=Miter+1

         call DTcg(Ncg)
         call Gram_Schmidt
         call psi_rho(0.5d0,0.5d0)
         if ( all( mask = rho==0.d0 ) ) then
            print *, "rho=0"
         endif
         call Hartree(rho,Vh,Hconv,iterVh)
         call Exc_Cor(XC,Exc)
         call Total_Energy

c iteration info.

         write(*,*) repeat('-',70)
         write(*,'(1x,"iter =",i4,5x,"Total Energy ="
     &               ,f15.8,5x,"Vh iteration =",i4)') Miter,Etot,iterVh

         write(*,'(:30(1x,5(i3,f9.4,2x)/))') ( p, esp(p), p=1,MST )

         call watch(DISP='on')

      end do Iteration

!---------------------------------------------------------------- Output

c LDA data

      if ( OC==1 ) then
         open(1,file=file_OUT,form='unformatted')
            call OUT_data
         close(1)
      end if

c LDA information

      open(1,file=LDA_Info,position='append')

      if ( IC==0 ) then
         write(1,*) "Box shape             = ", Box_shape
         write(1,*) "Radius of the box     = ", Rsize
         write(1,*) "Height of the box     = ", Zsize
         write(1,*) "Grid spacing          = ", H
         write(1,*) "Number of Grid points = ", ML," + ",MK," = ",ML+MK
         write(1,*)
         write(1,*) "Number of atoms = ", MI
         do ik=1,MKI
            write(1,'(1x,"Zatom(",i2,")       = ",i12)') ik, Zatom(ik)
         end do
         write(1,*)
         write(1,*) "Number of orbitals = ", MST
         write(1,*)
         write(1,*) "Ref. and max angular momentum",
     &              " and pseudo-core radius of PP"
         do ik=1,MKI
            write(1,'(1x,"(",i2,")  "," Ref, Max, Rps =",2i4,f8.3)')
     &      ik,Lref(ik),Mlps(ik),Rps(ik)
         end do
      end if

      write(1,*)
      write(1,*) "Exchange-Correlation = ",XC
      write(1,*)
      write(1,*) "Total number of iteration          = ", Miter
      write(1,*) "Number of Vh iteration (last time) = ", iterVh
      write(1,*)
      write(1,*) "Total energy = ", Etot
      write(1,*) "1-particle energies"

      write(1,'(:30(1x,5(i3,f9.4,2x)/))') ( p, esp(p), p=1,MST )
      write(1,'(//)')

      close(1)

      stop

      CONTAINS

!---------------------------------------------------- 1-particle density

      subroutine psi_rho(c1,c2)
      real(8),intent(IN) :: c1,c2
      rho = c1*rho + c2*matmul( psi**2, occ )
      return
      end subroutine psi_rho

!---------------------------------------- Gram-Schmidt orthogonalization

      subroutine Gram_Schmidt
      integer :: p,q
      real(8) :: ovrp(MST)
      psi(:,1)=psi(:,1)/sqrt(sum(psi(:,1)**2)*H**3)
      do p=2,MST
         q=p-1
         ovrp(1:q)=matmul( psi(:,p), psi(:,1:q) )*H**3
         psi(:,p)=psi(:,p)-matmul( psi(:,1:q),ovrp(1:q) )
         psi(:,p)=psi(:,p)/sqrt(sum(psi(:,p)**2)*H**3)
      end do
      return
      end subroutine Gram_Schmidt

!-----------------------------------------------------------Total energy

      subroutine Total_Energy
      integer :: p,ia,ib
      real(8) :: rab,htpsi(ML)
      do p=1,MST
         call hpsi(psi(:,p),htpsi) ; esp(p)=sum(psi(:,p)*htpsi)*H**3
      end do
      Etot = sum(occ*esp) + sum( rho*(-0.5d0*Vh-Vxc) )*H**3 + Exc
      do ia=1,MI
         do ib=1,ia-1
            rab=sqrt( sum((Rion(:,ia)-Rion(:,ib))**2) )
            Etot=Etot+Zps(Kion(ia))*Zps(Kion(ib))*E2/rab
         end do
      end do
      return
      end subroutine Total_Energy

      END PROGRAM Real_Space_DFT

!=======================================================================
!======================================= Conjugate-Gradient minimization

      SUBROUTINE DTcg(Ncg)
      use lda_data
      use lda_interfaces
      implicit none
      integer,intent(IN) :: Ncg

      integer :: iter,p,q
      real(8) :: sum0,xkHxk,xkxk,Rk,gkgk,xkpk,pkpk,pkHxk,pkHpk
      real(8) :: uk,alpha,Ak,Bk,Ck
      real(8),allocatable :: xk(:),hxk(:),gk(:),pk(:)

      allocate( xk(ML),hxk(ML),gk(ML),pk(ML) )

      orbital : do p=1,MST

         xk=psi(:,p) ; call hpsi(xk,hxk)

         xkHxk=sum(xk*hxk)*H**3 ; xkxk=1.d0 ; Rk=xkHxk/xkxk

         pk=0.d0 ; gkgk=1.d50

         Iteration : do iter=1,Ncg

            gk = 2*( hxk - Rk*xk )

            do q=1,p-1
               sum0=sum(psi(:,q)*gk)*H**3
               gk=gk-sum0*psi(:,q)
            end do

            sum0=sum(gk*gk)*H**3

            uk=sum0/gkgk ; pk=-gk+uk*pk ; gkgk=sum0

            xkpk = sum( xk*pk )*H**3
            pkpk = sum( pk*pk )*H**3
            pkHxk= sum( pk*hxk)*H**3

            call hpsi(pk,gk) ; pkHpk=sum(pk*gk)*H**3

            Ak=pkHpk*xkpk-pkHxk*pkpk
            Bk=pkHpk*xkxk-xkHxk*pkpk
            Ck=pkHxk*xkxk-xkHxk*xkpk
            alpha=(-Bk+sqrt(Bk*Bk-4*Ak*Ck))/(2*Ak)

            xk=xk+alpha*pk ; hxk=hxk+alpha*gk

            xkHxk=sum(xk*hxk)*H**3; xkxk=sum(xk*xk)*H**3; Rk=xkHxk/xkxk

         end do Iteration

         psi(:,p)=xk/sqrt(xkxk)

      end do orbital

      deallocate( xk,hxk,gk,pk )

      return

      END SUBROUTINE DTcg
