c=======================================================================
c================================================== Exchange-Correlation

      SUBROUTINE Exc_Cor(XC_in,Exc)
      use lda_data
      implicit none
      character(4),intent(IN)      :: XC_in
      real(8),intent(OUT),optional :: Exc

      real(8) :: tExc

      select case(XC_in)
         case('PZ','pz')     ; call Exc_Cor_Pz
         case('VWN','vwn')   ; call Exc_Cor_VWN
         case('LB94','lb94') ; call Exc_Cor_LB94
         case default
            write(*,*) "XC name "//XC_in//" is invalid." ; stop
      end select

      if ( present(Exc) ) Exc=tExc

      return

      CONTAINS

c=======================================================================
c=================================================== LDA (Perdew-Zunger)

      SUBROUTINE Exc_Cor_PZ

      real(8),parameter :: gm=-0.1423d0, b1=1.0529d0, b2=0.3334d0
      real(8),parameter :: AU=0.0311d0 , BU=-0.048d0
      real(8),parameter :: CU=0.002d0  , DU=-0.0116d0
      real(8),allocatable :: trho(:),rs(:),Vx(:),Vc(:),Ex(:),Ec(:)
      real(8) :: Cx
      integer :: i

      allocate( trho(ML),rs(ML),Vx(ML),Vc(ML),Ex(ML),Ec(ML) )

      trho=rho+1.d-20
      rs=(3.d0/(4*Pi*trho))**(1.d0/3.d0)/a_B

c Exchange

      Cx=3.d0/4.d0*(3.d0/(2*Pi))**(2.d0/3.d0)

      Vx=-4.d0/3.d0*Cx/rs ; Ex=-Cx/rs

c Correlation

      where ( rs > 1.d0 )
         Vc=gm*( 1.d0 + 7.d0/6.d0*b1*sqrt(rs) + 4.d0/3.d0*b2*rs )
     &        /( 1.d0 + b1*sqrt(rs) + b2*rs )**2
         Ec=gm/( 1.d0 + b1*sqrt(rs) + b2*rs )
      else where
         Vc=AU*log(rs) + (BU-AU/3.d0) + 2.d0/3.d0*CU*rs*log(rs)
     &        + (2.d0*DU-CU)/3.d0*rs
         Ec=AU*log(rs) + BU + CU*rs*log(rs) + DU*rs
      end where

      Vxc=2*Ry*(Vx+Vc) ; tExc=2*Ry*sum(trho*(Ex+Ec))*H**3

      deallocate( trho,rs,Vx,Vc,Ex,Ec )

      return

      END SUBROUTINE Exc_Cor_PZ

c=======================================================================
c===================================================== Vosko-Wilk-Nusair

      SUBROUTINE Exc_Cor_VWN

      real(8),parameter :: AA=0.0621814d0, d=-0.10498d0
      real(8),parameter :: b=3.72744d0   , c=12.9352d0
      real(8) :: Cx,Q,t1,t2,t3,x
      real(8),allocatable :: trho(:),rs(:),Vx(:),Ex(:),Vc(:),Ec(:)
      integer :: i

      allocate( trho(ML),rs(ML),Vx(ML),Ex(ML),Vc(ML),Ec(ML) )

      trho=rho+1.d-20
      rs=(3.d0/(4*Pi*trho))**(1.d0/3.d0)/a_B

c Exchange

      Cx=3.d0/4.d0*(3.d0/(2*Pi))**(2.d0/3.d0)

      Vx=-4.d0/3.d0*Cx/rs ; Ex=-Cx/rs

c Correlation

      Q = (4*c-b**2)**0.5d0

      do i=1,ML

         x  = sqrt( rs(i) )
         t1 =-(2*c+b*x)/(6*(c+x*(b+x))) + log( x**2/(c+x*(b+x)) )
         t2 = Q*x/(3*(b**2+Q**2+4*b*x+4*x**2)) + atan( Q/(b+2*x) )
         t3 = x*(2*c+b*x+d*(b+2*x))/(6*(d-x)*(c+x*(b+x)))
     &        + log( (d-x)**2/(c+x*(b+x)) )
         Vc(i) = AA*( t1 + 2*b/Q * t2
     &        - b*d/(d**2+b*d+c) * ( t3 + 2*(b+2*d)/Q * t2 ) )
         Vc(i)=Vc(i)/2.d0

         t1 = log( x**2/(x**2+b*x+c) )
         t2 = atan( Q/(2*x+b) )
         t3 = log( (x-d)**2/(x**2+b*x+c) )
         Ec(i)=AA*(t1+2*b/Q*t2-b*d/(x**2+b*x+c)*(t3+2*(b+2*d)/Q*t2))
         Ec(i)=Ec(i)/2.d0

      end do

      Vxc=2*Ry*(Vx+Vc) ; tExc=2*Ry*sum(trho*(Ex+Ec))*H**3

      deallocate( trho,rs,Vx,Ex,Vc,Ec )

      return

      END SUBROUTINE Exc_Cor_VWN

c================================================================== LB94

      SUBROUTINE Exc_Cor_LB94

      real(8),parameter :: beta=0.05d0
      real(8),allocatable :: trho(:),dnx(:),dny(:),dnz(:)
      real(8),allocatable :: w(:,:,:),x(:),Vx_GGA(:)
      real(8) :: sum1,sum2,sum3,r,rms
      integer :: i,j,ix,iy,iz

      allocate( trho(ML),dnx(ML),dny(ML),dnz(ML) )
      allocate( w(-Mx:Mx,-Mx:Mx,-Mx:Mx),x(ML),Vx_GGA(ML) )

c LDA

      call Exc_Cor_VWN

c Leeuwen-Baernds potential (LB94)

      trho = (rho+1.d-20)*a_B**3
      trho = trho/2.d0

      w=0.d0

      do i=1,ML
         w(Lx(i),Ly(i),Lz(i))=trho(i)
      end do

c 9-points finite-difference

      do i=1,ML
         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
         dnx(i) = ( bN1*( w(ix+1,iy,iz) - w(ix-1,iy,iz) )
     &             +bN2*( w(ix+2,iy,iz) - w(ix-2,iy,iz) )
     &             +bN3*( w(ix+3,iy,iz) - w(ix-3,iy,iz) )
     &             +bN4*( w(ix+4,iy,iz) - w(ix-4,iy,iz) ) )/H*a_B
         dny(i) = ( bN1*( w(ix,iy+1,iz) - w(ix,iy-1,iz) )
     &             +bN2*( w(ix,iy+2,iz) - w(ix,iy-2,iz) )
     &             +bN3*( w(ix,iy+3,iz) - w(ix,iy-3,iz) )
     &             +bN4*( w(ix,iy+4,iz) - w(ix,iy-4,iz) ) )/H*a_B
         dnz(i) = ( bN1*( w(ix,iy,iz+1) - w(ix,iy,iz-1) )
     &             +bN2*( w(ix,iy,iz+2) - w(ix,iy,iz-2) )
     &             +bN3*( w(ix,iy,iz+3) - w(ix,iy,iz-3) )
     &             +bN4*( w(ix,iy,iz+4) - w(ix,iy,iz-4) ) )/H*a_B
      end do

      x=sqrt(dnx**2+dny**2+dnz**2)/trho**(4.d0/3.d0)

      Vx_GGA = -beta*trho**(1.d0/3.d0)*x**2
     &       /( 1.d0+3.d0*beta*x*log(x+sqrt(1+x**2)) )

      Vxc=Vxc+2.d0*Ry*Vx_GGA

c Asymptotic Coulomb tail

!      rms=0.d0
!      do i=1,ML
!         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
!         r=(ix**2+iy**2+iz**2)*H**2
!         rms=rms+rho(i)*r
!      end do
!      rms=sqrt(rms*H**3)
!      do i=1,ML
!         ix=Lx(i) ; iy=Ly(i) ; iz=Lz(i)
!         r=sqrt((ix**2+iy**2+iz**2)*H**2)
!         if ( r>rms ) Vxc(i)=-E2/r
!      end do

      deallocate( trho,dnx,dny,dnz,w,x,Vx_GGA )

      return

      END SUBROUTINE Exc_Cor_LB94

      END SUBROUTINE Exc_Cor
